from .atlas import graph_atlas as graph_atlas, graph_atlas_g as graph_atlas_g
from .classic import (
    balanced_tree as balanced_tree,
    barbell_graph as barbell_graph,
    binomial_tree as binomial_tree,
    complete_graph as complete_graph,
    complete_multipartite_graph as complete_multipartite_graph,
    circular_ladder_graph as circular_ladder_graph,
    circulant_graph as circulant_graph,
    cycle_graph as cycle_graph,
    dorogovtsev_goltsev_mendes_graph as dorogovtsev_goltsev_mendes_graph,
    empty_graph as empty_graph,
    full_rary_tree as full_rary_tree,
    ladder_graph as ladder_graph,
    lollipop_graph as lollipop_graph,
    null_graph as null_graph,
    path_graph as path_graph,
    star_graph as star_graph,
    trivial_graph as trivial_graph,
    turan_graph as turan_graph,
    wheel_graph as wheel_graph,
)
from .cographs import random_cograph as random_cograph
from .community import (
    caveman_graph as caveman_graph,
    connected_caveman_graph as connected_caveman_graph,
    relaxed_caveman_graph as relaxed_caveman_graph,
    random_partition_graph as random_partition_graph,
    planted_partition_graph as planted_partition_graph,
    gaussian_random_partition_graph as gaussian_random_partition_graph,
    ring_of_cliques as ring_of_cliques,
    windmill_graph as windmill_graph,
    stochastic_block_model as stochastic_block_model,
    LFR_benchmark_graph as LFR_benchmark_graph,
)
from .degree_seq import (
    configuration_model as configuration_model,
    directed_configuration_model as directed_configuration_model,
    expected_degree_graph as expected_degree_graph,
    havel_hakimi_graph as havel_hakimi_graph,
    directed_havel_hakimi_graph as directed_havel_hakimi_graph,
    degree_sequence_tree as degree_sequence_tree,
    random_degree_sequence_graph as random_degree_sequence_graph,
)
from .directed import (
    gn_graph as gn_graph,
    gnc_graph as gnc_graph,
    gnr_graph as gnr_graph,
    random_k_out_graph as random_k_out_graph,
    scale_free_graph as scale_free_graph,
)
from .duplication import (
    partial_duplication_graph as partial_duplication_graph,
    duplication_divergence_graph as duplication_divergence_graph,
)
from .ego import ego_graph as ego_graph
from .expanders import (
    margulis_gabber_galil_graph as margulis_gabber_galil_graph,
    chordal_cycle_graph as chordal_cycle_graph,
    paley_graph as paley_graph,
)
from .geometric import (
    geometric_edges as geometric_edges,
    geographical_threshold_graph as geographical_threshold_graph,
    navigable_small_world_graph as navigable_small_world_graph,
    random_geometric_graph as random_geometric_graph,
    soft_random_geometric_graph as soft_random_geometric_graph,
    thresholded_random_geometric_graph as thresholded_random_geometric_graph,
    waxman_graph as waxman_graph,
)
from .internet_as_graphs import random_internet_as_graph as random_internet_as_graph
from .intersection import (
    uniform_random_intersection_graph as uniform_random_intersection_graph,
    k_random_intersection_graph as k_random_intersection_graph,
    general_random_intersection_graph as general_random_intersection_graph,
)
from .interval_graph import interval_graph as interval_graph
from .joint_degree_seq import (
    is_valid_joint_degree as is_valid_joint_degree,
    is_valid_directed_joint_degree as is_valid_directed_joint_degree,
    joint_degree_graph as joint_degree_graph,
    directed_joint_degree_graph as directed_joint_degree_graph,
)
from .lattice import (
    grid_2d_graph as grid_2d_graph,
    grid_graph as grid_graph,
    hypercube_graph as hypercube_graph,
    triangular_lattice_graph as triangular_lattice_graph,
    hexagonal_lattice_graph as hexagonal_lattice_graph,
)
from .line import line_graph as line_graph, inverse_line_graph as inverse_line_graph
from .mycielski import mycielskian as mycielskian, mycielski_graph as mycielski_graph
from .nonisomorphic_trees import (
    nonisomorphic_trees as nonisomorphic_trees,
    number_of_nonisomorphic_trees as number_of_nonisomorphic_trees,
)
from .random_clustered import random_clustered_graph as random_clustered_graph
from .random_graphs import (
    fast_gnp_random_graph as fast_gnp_random_graph,
    gnp_random_graph as gnp_random_graph,
    dense_gnm_random_graph as dense_gnm_random_graph,
    gnm_random_graph as gnm_random_graph,
    erdos_renyi_graph as erdos_renyi_graph,
    binomial_graph as binomial_graph,
    newman_watts_strogatz_graph as newman_watts_strogatz_graph,
    watts_strogatz_graph as watts_strogatz_graph,
    connected_watts_strogatz_graph as connected_watts_strogatz_graph,
    random_regular_graph as random_regular_graph,
    barabasi_albert_graph as barabasi_albert_graph,
    dual_barabasi_albert_graph as dual_barabasi_albert_graph,
    extended_barabasi_albert_graph as extended_barabasi_albert_graph,
    powerlaw_cluster_graph as powerlaw_cluster_graph,
    random_lobster as random_lobster,
    random_shell_graph as random_shell_graph,
    random_powerlaw_tree as random_powerlaw_tree,
    random_powerlaw_tree_sequence as random_powerlaw_tree_sequence,
    random_kernel_graph as random_kernel_graph,
)
from .small import (
    make_small_graph as make_small_graph,
    LCF_graph as LCF_graph,
    bull_graph as bull_graph,
    chvatal_graph as chvatal_graph,
    cubical_graph as cubical_graph,
    desargues_graph as desargues_graph,
    diamond_graph as diamond_graph,
    dodecahedral_graph as dodecahedral_graph,
    frucht_graph as frucht_graph,
    heawood_graph as heawood_graph,
    hoffman_singleton_graph as hoffman_singleton_graph,
    house_graph as house_graph,
    house_x_graph as house_x_graph,
    icosahedral_graph as icosahedral_graph,
    krackhardt_kite_graph as krackhardt_kite_graph,
    moebius_kantor_graph as moebius_kantor_graph,
    octahedral_graph as octahedral_graph,
    pappus_graph as pappus_graph,
    petersen_graph as petersen_graph,
    sedgewick_maze_graph as sedgewick_maze_graph,
    tetrahedral_graph as tetrahedral_graph,
    truncated_cube_graph as truncated_cube_graph,
    truncated_tetrahedron_graph as truncated_tetrahedron_graph,
    tutte_graph as tutte_graph,
)
from .social import (
    karate_club_graph as karate_club_graph,
    davis_southern_women_graph as davis_southern_women_graph,
    florentine_families_graph as florentine_families_graph,
    les_miserables_graph as les_miserables_graph,
)
from .sudoku import sudoku_graph as sudoku_graph
from .spectral_graph_forge import spectral_graph_forge as spectral_graph_forge
from .stochastic import stochastic_graph as stochastic_graph
from .trees import prefix_tree as prefix_tree, random_tree as random_tree, prefix_tree_recursive as prefix_tree_recursive
from .triads import triad_graph as triad_graph



from typing import Callable, ClassVar, TypeVar
from numpy.random import RandomState
from .base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .utils.validation import (
    check_is_fitted as check_is_fitted,
    check_non_negative as check_non_negative,
)
from numpy import ndarray
from scipy.linalg import svd as svd
from numbers import Integral as Integral, Real as Real
from .utils.extmath import safe_sparse_dot as safe_sparse_dot
from .utils import check_random_state as check_random_state
from .utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.sparse import spmatrix
from scipy.fftpack import fft as fft, ifft as ifft
from .metrics.pairwise import (
    pairwise_kernels as pairwise_kernels,
    KERNEL_PARAMS as KERNEL_PARAMS,
    PAIRWISE_KERNEL_FUNCTIONS as PAIRWISE_KERNEL_FUNCTIONS,
)
from ._typing import Float, Int, MatrixLike, ArrayLike

SkewedChi2Sampler_Self = TypeVar("SkewedChi2Sampler_Self", bound="SkewedChi2Sampler")
PolynomialCountSketch_Self = TypeVar(
    "PolynomialCountSketch_Self", bound="PolynomialCountSketch"
)
AdditiveChi2Sampler_Self = TypeVar(
    "AdditiveChi2Sampler_Self", bound="AdditiveChi2Sampler"
)
RBFSampler_Self = TypeVar("RBFSampler_Self", bound="RBFSampler")
Nystroem_Self = TypeVar("Nystroem_Self", bound="Nystroem")

import warnings

import numpy as np
import scipy.sparse as sp


class PolynomialCountSketch(
    ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator
):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    bitHash_: ndarray = ...
    indexHash_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        gamma: Float = 1.0,
        degree: Int = 2,
        coef0: Int = 0,
        n_components: Int = 100,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: PolynomialCountSketch_Self,
        X: MatrixLike | ArrayLike,
        y: None | MatrixLike | ArrayLike = None,
    ) -> PolynomialCountSketch_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...


class RBFSampler(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    random_weights_: ndarray = ...
    random_offset_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        gamma: float | str = 1.0,
        n_components: Int = 100,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: RBFSampler_Self, X: MatrixLike, y: None | MatrixLike | ArrayLike = None
    ) -> RBFSampler_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...


class SkewedChi2Sampler(
    ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator
):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    random_offset_: ndarray = ...
    random_weights_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        skewedness: Float = 1.0,
        n_components: Int = 100,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: SkewedChi2Sampler_Self,
        X: MatrixLike,
        y: None | MatrixLike | ArrayLike = None,
    ) -> SkewedChi2Sampler_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...


class AdditiveChi2Sampler(TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    sample_interval_: float = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, *, sample_steps: Int = 2, sample_interval: None | Float = None
    ) -> None:
        ...

    def fit(
        self: AdditiveChi2Sampler_Self,
        X: MatrixLike,
        y: None | MatrixLike | ArrayLike = None,
    ) -> AdditiveChi2Sampler_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray | spmatrix:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...


class Nystroem(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    normalization_: ndarray = ...
    component_indices_: ndarray = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        kernel: str | Callable = "rbf",
        *,
        gamma: None | Float = None,
        coef0: None | Float = None,
        degree: None | Float = None,
        kernel_params: None | dict = None,
        n_components: Int = 100,
        random_state: RandomState | None | Int = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: Nystroem_Self, X: MatrixLike, y: None | MatrixLike | ArrayLike = None
    ) -> Nystroem_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...



from collections.abc import Iterable
from typing import Any
from itertools import chain
from ..classes.graph import Graph

__all__ = ["edge_boundary", "node_boundary"]

def edge_boundary(
    G: Graph,
    nbunch1: Iterable,
    nbunch2: Iterable | None = None,
    data: bool | Any = False,
    keys: bool = False,
    default: Any = None,
): ...
def node_boundary(
    G: Graph, nbunch1: Iterable, nbunch2: Iterable | None = None
) -> set: ...



from collections.abc import Iterable
from numpy.typing import ArrayLike

from ..classes.graph import Graph
from ..utils import not_implemented_for

from .distance_measures import diameter

__all__ = [
    "is_distance_regular",
    "is_strongly_regular",
    "intersection_array",
    "global_parameters",
]

def is_distance_regular(G: Graph) -> bool: ...
def global_parameters(b: ArrayLike, c: ArrayLike) -> Iterable: ...
def intersection_array(G: Graph): ...

# TODO There is a definition for directed strongly regular graphs.

def is_strongly_regular(G: Graph) -> bool: ...



import itertools
import time
from xml.etree.ElementTree import (
    Element,
    ElementTree,
    SubElement,
    register_namespace,
    tostring,
)

from ..classes.graph import Graph
from ..utils import open_file

__all__ = ["write_gexf", "read_gexf", "relabel_gexf_graph", "generate_gexf"]

def write_gexf(
    G: Graph, path, encoding="utf-8", prettyprint=True, version="1.2draft"
): ...
def generate_gexf(G: Graph, encoding="utf-8", prettyprint=True, version="1.2draft"): ...
def read_gexf(path, node_type=None, relabel=False, version="1.2draft"): ...

class GEXF:
    versions: dict = ...
    d: dict = ...
    versions["1.1draft"] = ...
    d: dict = ...
    versions["1.2draft"] = ...

    def construct_types(self): ...

    # http://www.w3.org/TR/xmlschema-2/#boolean
    convert_bool: dict = ...

    def set_version(self, version): ...

class GEXFWriter(GEXF):
    # class for writing GEXF format files
    # use write_gexf() function
    def __init__(
        self, graph=None, encoding="utf-8", prettyprint=True, version="1.2draft"
    ): ...
    def __str__(self): ...
    def add_graph(self, G): ...
    def add_nodes(self, G: Graph, graph_element): ...
    def add_edges(self, G: Graph, graph_element): ...
    def add_attributes(self, node_or_edge, xml_obj, data, default): ...
    def get_attr_id(self, title, attr_type, edge_or_node, default, mode): ...
    def add_viz(self, element, node_data): ...
    def add_parents(self, node_element, node_data): ...
    def add_slices(self, node_or_edge_element, node_or_edge_data): ...
    def add_spells(self, node_or_edge_element, node_or_edge_data): ...
    def alter_graph_mode_timeformat(self, start_or_end): ...
    def write(self, fh): ...
    def indent(self, elem, level=0): ...

class GEXFReader(GEXF):
    # Class to read GEXF format files
    # use read_gexf() function
    def __init__(self, node_type=None, version="1.2draft"): ...
    def __call__(self, stream): ...
    def make_graph(self, graph_xml): ...
    def add_node(self, G: Graph, node_xml, node_attr, node_pid=None): ...
    def add_start_end(self, data, xml): ...
    def add_viz(self, data, node_xml): ...
    def add_parents(self, data, node_xml): ...
    def add_slices(self, data, node_or_edge_xml): ...
    def add_spells(self, data, node_or_edge_xml): ...
    def add_edge(self, G: Graph, edge_element, edge_attr): ...
    def decode_attr_elements(self, gexf_keys, obj_xml): ...
    def find_gexf_attributes(self, attributes_element): ...

def relabel_gexf_graph(G: Graph): ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import subprocess

def use(app: str | None = None, gl: str | None = None): ...
def run_subprocess(command: ArrayLike, return_code: bool = False, **kwargs) -> tuple[str, str, int]: ...
def test(*args, **kwargs): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# Adapted from PyQtGraph
import sys
from . import ptime
from .. import config

class Profiler(object):

    _profilers = ...

    _depth: int = ...
    _msgs: list = ...
    # set this flag to disable all or individual profilers at runtime
    disable: bool = ...
    ...

    _disabled_profiler = ...

    def __new__(cls, msg=None, disabled="env", delayed=True): ...
    def __call__(self, msg=None, *args): ...
    def mark(self, msg=None): ...
    def _new_msg(self, msg, *args): ...
    def __del__(self): ...
    def finish(self, msg=None): ...
    def flush(self): ...



import sys
from typing import (
    Any,
    Callable,
    List,
    Mapping,
    Optional,
    Protocol,
    Sequence,
    Type,
    TypeVar,
    Union,
    overload,
)

from django.db.models import Manager, QuerySet
from django.db.models.base import Model
from django.http import HttpRequest
from django.http.response import HttpResponse as HttpResponse
from django.http.response import (
    HttpResponsePermanentRedirect as HttpResponsePermanentRedirect,
)
from django.http.response import HttpResponseRedirect as HttpResponseRedirect

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

def render_to_response(
    template_name: Union[str, Sequence[str]],
    context: Optional[Mapping[str, Any]] = ...,
    content_type: Optional[str] = ...,
    status: Optional[int] = ...,
    using: Optional[str] = ...,
) -> HttpResponse: ...
def render(
    request: HttpRequest,
    template_name: Union[str, Sequence[str]],
    context: Optional[Mapping[str, Any]] = ...,
    content_type: Optional[str] = ...,
    status: Optional[int] = ...,
    using: Optional[str] = ...,
) -> HttpResponse: ...

class SupportsGetAbsoluteUrl(Protocol): ...

@overload
def redirect(
    to: Union[Callable[..., Any], str, SupportsGetAbsoluteUrl],
    *args: Any,
    permanent: Literal[True],
    **kwargs: Any
) -> HttpResponsePermanentRedirect: ...
@overload
def redirect(
    to: Union[Callable[..., Any], str, SupportsGetAbsoluteUrl],
    *args: Any,
    permanent: Literal[False],
    **kwargs: Any
) -> HttpResponseRedirect: ...
@overload
def redirect(
    to: Union[Callable[..., Any], str, SupportsGetAbsoluteUrl],
    *args: Any,
    permanent: bool = ...,
    **kwargs: Any
) -> Union[HttpResponseRedirect, HttpResponsePermanentRedirect]: ...

_T = TypeVar("_T", bound=Model)

def get_object_or_404(
    klass: Union[Type[_T], Manager[_T], QuerySet[_T]], *args: Any, **kwargs: Any
) -> _T: ...
def get_list_or_404(
    klass: Union[Type[_T], Manager[_T], QuerySet[_T]], *args: Any, **kwargs: Any
) -> List[_T]: ...
def resolve_url(
    to: Union[Callable[..., Any], Model, str], *args: Any, **kwargs: Any
) -> str: ...



# Stubs for django.conf.urls (Python 3.5)
from typing import Any, Callable, Dict, List, Optional, Tuple, Union, overload

from django.http.response import HttpResponse, HttpResponseBase
from django.urls import URLPattern, URLResolver

handler400: Union[str, Callable[..., HttpResponse]] = ...
handler403: Union[str, Callable[..., HttpResponse]] = ...
handler404: Union[str, Callable[..., HttpResponse]] = ...
handler500: Union[str, Callable[..., HttpResponse]] = ...

IncludedURLConf = Tuple[List[URLResolver], Optional[str], Optional[str]]

def include(arg: Any, namespace: str = ..., app_name: str = ...) -> IncludedURLConf: ...
@overload
def url(
    regex: str,
    view: Callable[..., HttpResponseBase],
    kwargs: Dict[str, Any] = ...,
    name: str = ...,
) -> URLPattern: ...
@overload
def url(
    regex: str, view: IncludedURLConf, kwargs: Dict[str, Any] = ..., name: str = ...
) -> URLResolver: ...
@overload
def url(
    regex: str,
    view: List[Union[URLResolver, str]],
    kwargs: Dict[str, Any] = ...,
    name: str = ...,
) -> URLResolver: ...



from .adjlist import (
    generate_adjlist as generate_adjlist,
    write_adjlist as write_adjlist,
    parse_adjlist as parse_adjlist,
    read_adjlist as read_adjlist,
)
from .multiline_adjlist import (
    generate_multiline_adjlist as generate_multiline_adjlist,
    write_multiline_adjlist as write_multiline_adjlist,
    parse_multiline_adjlist as parse_multiline_adjlist,
    read_multiline_adjlist as read_multiline_adjlist,
)
from .edgelist import (
    generate_edgelist as generate_edgelist,
    write_edgelist as write_edgelist,
    parse_edgelist as parse_edgelist,
    read_edgelist as read_edgelist,
    read_weighted_edgelist as read_weighted_edgelist,
    write_weighted_edgelist as write_weighted_edgelist,
)
from .gpickle import read_gpickle as read_gpickle, write_gpickle as write_gpickle
from .pajek import (
    read_pajek as read_pajek,
    parse_pajek as parse_pajek,
    generate_pajek as generate_pajek,
    write_pajek as write_pajek,
)
from .leda import read_leda as read_leda, parse_leda as parse_leda
from .sparse6 import (
    from_sparse6_bytes as from_sparse6_bytes,
    read_sparse6 as read_sparse6,
    to_sparse6_bytes as to_sparse6_bytes,
    write_sparse6 as write_sparse6,
)
from .graph6 import (
    from_graph6_bytes as from_graph6_bytes,
    read_graph6 as read_graph6,
    to_graph6_bytes as to_graph6_bytes,
    write_graph6 as write_graph6,
)
from .gml import read_gml as read_gml, parse_gml as parse_gml, generate_gml as generate_gml, write_gml as write_gml
from .graphml import (
    write_graphml as write_graphml,
    read_graphml as read_graphml,
    generate_graphml as generate_graphml,
    write_graphml_xml as write_graphml_xml,
    write_graphml_lxml as write_graphml_lxml,
    parse_graphml as parse_graphml,
    GraphMLWriter as GraphMLWriter,
    GraphMLReader as GraphMLReader,
)
from .gexf import (
    write_gexf as write_gexf,
    read_gexf as read_gexf,
    relabel_gexf_graph as relabel_gexf_graph,
    generate_gexf as generate_gexf,
)
from .nx_shp import read_shp as read_shp, write_shp as write_shp
from .json_graph import *
from .text import forest_str as forest_str



from typing import Literal
from scipy.special import xlogy as xlogy
from ..exceptions import UndefinedMetricWarning as UndefinedMetricWarning
from numpy import ndarray
from .._typing import MatrixLike, ArrayLike, Float
from ..utils.validation import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
    check_scalar as check_scalar,
    column_or_1d as column_or_1d,
)

# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Mathieu Blondel <mathieu@mblondel.org>
#          Olivier Grisel <olivier.grisel@ensta.org>
#          Arnaud Joly <a.joly@ulg.ac.be>
#          Jochen Wersdorfer <jochen@wersdoerfer.de>
#          Lars Buitinck
#          Joel Nothman <joel.nothman@gmail.com>
#          Karan Desai <karandesai281196@gmail.com>
#          Noel Dawe <noel@dawe.me>
#          Manoj Kumar <manojkumarsivaraj334@gmail.com>
#          Michael Eickenberg <michael.eickenberg@gmail.com>
#          Konstantin Shmelkov <konstantin.shmelkov@polytechnique.edu>
#          Christian Lorentzen <lorentzen.ch@gmail.com>
#          Ashutosh Hathidara <ashutoshhathidara98@gmail.com>
#          Uttam kumar <bajiraouttamsinha@gmail.com>
#          Sylvain Marie <sylvain.marie@se.com>
#          Ohad Michel <ohadmich@gmail.com>
# License: BSD 3 clause

import numbers
import warnings

import numpy as np


__ALL__: list = ...


def mean_absolute_error(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
) -> ndarray | Float:
    ...


def mean_pinball_loss(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    alpha: float = 0.5,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
) -> ndarray | Float:
    ...


def mean_absolute_percentage_error(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
) -> ndarray | Float:
    ...


def mean_squared_error(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
    squared: bool = True,
) -> ndarray | Float:
    ...


def mean_squared_log_error(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
    squared: bool = True,
) -> float | ndarray:
    ...


def median_absolute_error(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
    sample_weight: None | ArrayLike = None,
) -> ndarray | Float:
    ...


def explained_variance_score(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    multioutput: Literal[
        "raw_values", "uniform_average", "variance_weighted", "uniform_average"
    ]
    | ArrayLike = "uniform_average",
    force_finite: bool = True,
) -> float | ndarray:
    ...


def r2_score(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    multioutput: Literal[
        "raw_values", "uniform_average", "variance_weighted", "uniform_average"
    ]
    | None
    | ArrayLike = "uniform_average",
    force_finite: bool = True,
) -> ndarray | Float:
    ...


def max_error(y_true: ArrayLike, y_pred: ArrayLike) -> float:
    ...


def mean_tweedie_deviance(
    y_true: ArrayLike,
    y_pred: ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    power: Float = 0,
) -> Float:
    ...


def mean_poisson_deviance(
    y_true: ArrayLike, y_pred: ArrayLike, *, sample_weight: None | ArrayLike = None
) -> Float:
    ...


def mean_gamma_deviance(
    y_true: ArrayLike, y_pred: ArrayLike, *, sample_weight: None | ArrayLike = None
) -> float:
    ...


def d2_tweedie_score(
    y_true: ArrayLike,
    y_pred: ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    power: Float = 0,
) -> float | ndarray:
    ...


def d2_pinball_score(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    alpha: Float = 0.5,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
) -> float | ndarray:
    ...


def d2_absolute_error_score(
    y_true: MatrixLike | ArrayLike,
    y_pred: MatrixLike | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    multioutput: ArrayLike
    | Literal["raw_values", "uniform_average", "uniform_average"] = "uniform_average",
) -> float | ndarray:
    ...



from typing import Any, Callable, ClassVar, Literal, Set, TypeVar
from scipy import sparse as sparse
from joblib import Memory
from ..metrics import DistanceMetric as DistanceMetric
from scipy.cluster import hierarchy as hierarchy
from ..metrics._dist_metrics import METRIC_MAPPING as METRIC_MAPPING
from scipy.sparse.csgraph import connected_components as connected_components
from ._feature_agglomeration import AgglomerationTransform
from heapq import (
    heapify as heapify,
    heappop as heappop,
    heappush as heappush,
    heappushpop as heappushpop,
)
from ..metrics.pairwise import paired_distances as paired_distances
from ..utils.validation import check_memory as check_memory
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
    HasMethods as HasMethods,
)
from numbers import Integral as Integral, Real as Real
from ..utils._fast_dict import IntFloatDict as IntFloatDict
from ..base import BaseEstimator, ClusterMixin, ClassNamePrefixFeaturesOutMixin
from ..utils import check_array as check_array
from .._typing import MatrixLike, Int, ArrayLike, Float

FeatureAgglomeration_Self = TypeVar(
    "FeatureAgglomeration_Self", bound="FeatureAgglomeration"
)
AgglomerativeClustering_Self = TypeVar(
    "AgglomerativeClustering_Self", bound="AgglomerativeClustering"
)

import warnings

import numpy as np


###############################################################################
# Hierarchical tree building functions


def ward_tree(
    X: MatrixLike,
    *,
    connectivity: None | MatrixLike = None,
    n_clusters: None | Int = None,
    return_distance: bool = False,
) -> tuple[ndarray, int, int, ndarray | None, ndarray]:
    ...


# single average and complete linkage
def linkage_tree(
    X: MatrixLike,
    connectivity: None | MatrixLike = None,
    n_clusters: None | Int = None,
    linkage: Literal["complete", "average", "complete", "single"] = "complete",
    affinity: str | Callable = "euclidean",
    return_distance: bool = False,
) -> tuple[ndarray, int, int, ndarray | None, ndarray]:
    ...


_TREE_BUILDERS = ...


###############################################################################


class AgglomerativeClustering(ClusterMixin, BaseEstimator):
    distances_: ArrayLike = ...
    children_: ArrayLike = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_connected_components_: int = ...
    n_leaves_: int = ...
    labels_: ndarray = ...
    n_clusters_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: None | int = 2,
        *,
        affinity: str | Callable = "deprecated",  # TODO(1.4): Remove
        metric: None | str | Callable = None,  # TODO(1.4): Set to "euclidean"
        memory: None | Memory | str = None,
        connectivity: None | ArrayLike | Callable = None,
        compute_full_tree: Literal["auto", "auto"] | bool = "auto",
        linkage: Literal["ward", "complete", "average", "single", "ward"] = "ward",
        distance_threshold: None | Float = None,
        compute_distances: bool = False,
    ) -> None:
        ...

    def fit(
        self: AgglomerativeClustering_Self, X: MatrixLike, y: Any = None
    ) -> AgglomerativeClustering_Self:
        ...

    def fit_predict(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...


class FeatureAgglomeration(
    ClassNamePrefixFeaturesOutMixin, AgglomerativeClustering, AgglomerationTransform
):
    distances_: ArrayLike = ...
    children_: ArrayLike = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_connected_components_: int = ...
    n_leaves_: int = ...
    labels_: ArrayLike = ...
    n_clusters_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: None | int = 2,
        *,
        affinity: str | Callable = "deprecated",  # TODO(1.4): Remove
        metric: None | str | Callable = None,  # TODO(1.4): Set to "euclidean"
        memory: None | Memory | str = None,
        connectivity: None | ArrayLike | Callable = None,
        compute_full_tree: Literal["auto", "auto"] | bool = "auto",
        linkage: Literal["ward", "complete", "average", "single", "ward"] = "ward",
        pooling_func: Callable = ...,
        distance_threshold: None | Float = None,
        compute_distances: bool = False,
    ) -> None:
        ...

    def fit(
        self: FeatureAgglomeration_Self, X: MatrixLike, y: Any = None
    ) -> FeatureAgglomeration_Self:
        ...

    def fit_predict(self):
        ...



from ..classes.digraph import DiGraph
from numpy.typing import ArrayLike

import heapq
from collections import deque
from functools import partial
from itertools import chain, product, starmap
from math import gcd

from ..classes.graph import Graph
from ..utils import arbitrary_element, not_implemented_for, pairwise

__all__ = [
    "descendants",
    "ancestors",
    "topological_sort",
    "lexicographical_topological_sort",
    "all_topological_sorts",
    "topological_generations",
    "is_directed_acyclic_graph",
    "is_aperiodic",
    "transitive_closure",
    "transitive_closure_dag",
    "transitive_reduction",
    "antichains",
    "dag_longest_path",
    "dag_longest_path_length",
    "dag_to_branching",
]

chaini = ...

def descendants(G: Graph, source): ...
def ancestors(G: Graph, source): ...
def has_cycle(G: Graph): ...
def is_directed_acyclic_graph(G: Graph) -> bool: ...
def topological_generations(G: Graph): ...
def topological_sort(G: Graph): ...
def lexicographical_topological_sort(G: Graph, key=None): ...
def all_topological_sorts(G: Graph): ...
def is_aperiodic(G: Graph) -> bool: ...
def transitive_closure(G: Graph, reflexive=False): ...
def transitive_closure_dag(G: Graph, topo_order: ArrayLike | tuple | None = None): ...
def transitive_reduction(G: Graph): ...
def antichains(G: Graph, topo_order: ArrayLike | tuple | None = None): ...
def dag_longest_path(
    G: Graph,
    weight: str = "weight",
    default_weight: int = 1,
    topo_order: ArrayLike | tuple | None = None,
) -> ArrayLike: ...
def dag_longest_path_length(
    G: Graph, weight: str = "weight", default_weight: int = 1
) -> int: ...
def root_to_leaf_paths(G: Graph): ...
def dag_to_branching(G: Graph) -> DiGraph: ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from ..base import BaseEstimator
from numpy import ufunc, ndarray
from ..utils._param_validation import StrOptions as StrOptions
from ..utils.metaestimators import available_if as available_if
from ..base import TransformerMixin
from pandas.core.frame import DataFrame
from .._typing import MatrixLike, ArrayLike
from ..utils.validation import check_array as check_array

FunctionTransformer_Self = TypeVar(
    "FunctionTransformer_Self", bound="FunctionTransformer"
)

import warnings

import numpy as np


class FunctionTransformer(TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        func: ufunc | None | Callable = None,
        inverse_func: None | ufunc | Callable = None,
        *,
        validate: bool = False,
        accept_sparse: bool = False,
        check_inverse: bool = True,
        feature_names_out: None | str | Callable = None,
        kw_args: None | dict = None,
        inv_kw_args: None | dict = None,
    ) -> None:
        ...

    def fit(
        self: FunctionTransformer_Self, X: list[str] | MatrixLike, y: Any = None
    ) -> FunctionTransformer_Self:
        ...

    def transform(
        self, X: list[str] | MatrixLike
    ) -> ndarray | DataFrame | list[dict[str, int]]:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def __sklearn_is_fitted__(self):
        ...

    def set_output(
        self, *, transform: None | Literal["default", "pandas"] = None
    ) -> BaseEstimator:
        ...



from .attrmatrix import attr_matrix as attr_matrix, attr_sparse_matrix as attr_sparse_matrix
from . import attrmatrix as attrmatrix
from .spectrum import (
    laplacian_spectrum as laplacian_spectrum,
    adjacency_spectrum as adjacency_spectrum,
    modularity_spectrum as modularity_spectrum,
    normalized_laplacian_spectrum as normalized_laplacian_spectrum,
    bethe_hessian_spectrum as bethe_hessian_spectrum,
)
from . import spectrum as spectrum
from .graphmatrix import incidence_matrix as incidence_matrix, adj_matrix as adj_matrix, adjacency_matrix as adjacency_matrix
from . import graphmatrix as graphmatrix
from .laplacianmatrix import (
    laplacian_matrix as laplacian_matrix,
    normalized_laplacian_matrix as normalized_laplacian_matrix,
    total_spanning_tree_weight as total_spanning_tree_weight,
    directed_laplacian_matrix as directed_laplacian_matrix,
    directed_combinatorial_laplacian_matrix as directed_combinatorial_laplacian_matrix,
)
from . import laplacianmatrix as laplacianmatrix
from .algebraicconnectivity import (
    algebraic_connectivity as algebraic_connectivity,
    fiedler_vector as fiedler_vector,
    spectral_ordering as spectral_ordering,
)
from .modularitymatrix import modularity_matrix as modularity_matrix, directed_modularity_matrix as directed_modularity_matrix
from . import modularitymatrix as modularitymatrix
from .bethehessianmatrix import bethe_hessian_matrix as bethe_hessian_matrix
from . import bethehessianmatrix as bethehessianmatrix



from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["is_regular", "is_k_regular", "k_factor"]

def is_regular(G: Graph) -> bool: ...
def is_k_regular(G: Graph, k) -> bool: ...
def k_factor(G: Graph, k, matching_weight="weight"): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.special._test_round, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def assert_(val, msg) -> typing.Any:
    '\n    Assert that works in release mode.\n    Accepts callable msg to allow deferring evaluation until failure.\n\n    The Python built-in ``assert`` does not work when executing code in\n    optimized mode (the ``-O`` flag) - no byte-code is generated for it.\n\n    For documentation on usage, refer to the Python documentation.\n\n    '
    ...

def have_fenv() -> typing.Any:
    ...

def random_double() -> typing.Any:
    ...

def test_add_round() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




from numpy import ndarray
from numpy.typing import NDArray, ArrayLike
from typing import Literal
from itertools import combinations_with_replacement
from warnings import warn

import numpy as np
from scipy import ndimage as ndi
from scipy import spatial, stats

from .._shared.filters import gaussian
from .._shared.utils import _supported_float_type, safe_as_int
from ..transform import integral_image
from .peak import peak_local_max
from .util import _prepare_grayscale_input_2D, _prepare_grayscale_input_nD

def _compute_derivatives(image, mode="constant", cval=0): ...
def structure_tensor(
    image: NDArray,
    sigma: float | ArrayLike = 1,
    mode: Literal["constant", "reflect", "wrap", "nearest", "mirror"] = "constant",
    cval: float = 0,
    order: Literal["rc", "xy"] | None = None,
) -> ArrayLike: ...
def hessian_matrix(
    image: NDArray,
    sigma: float = 1,
    mode: Literal["constant", "reflect", "wrap", "nearest", "mirror"] = "constant",
    cval: float = 0,
    order: Literal["rc", "xy"] = "rc",
) -> ArrayLike: ...
def hessian_matrix_det(
    image: NDArray, sigma: float = 1, approximate: bool = True
) -> ArrayLike: ...
def _image_orthogonal_matrix22_eigvals(M00, M01, M11): ...
def _symmetric_compute_eigenvalues(S_elems): ...
def _symmetric_image(S_elems): ...
def structure_tensor_eigenvalues(A_elems: ArrayLike) -> NDArray: ...
def structure_tensor_eigvals(
    Axx: NDArray, Axy: NDArray, Ayy: NDArray
) -> tuple[NDArray, NDArray]: ...
def hessian_matrix_eigvals(H_elems: ArrayLike) -> NDArray: ...
def shape_index(
    image,
    sigma: float = 1,
    mode: Literal["constant", "reflect", "wrap", "nearest", "mirror"] = "constant",
    cval: float = 0,
) -> NDArray: ...
def corner_kitchen_rosenfeld(
    image,
    mode: Literal["constant", "reflect", "wrap", "nearest", "mirror"] = "constant",
    cval: float = 0,
) -> NDArray: ...
def corner_harris(
    image,
    method: Literal["k", "eps"] = "k",
    k: float = 0.05,
    eps: float = 1e-6,
    sigma: float = 1,
) -> NDArray: ...
def corner_shi_tomasi(image, sigma: float = 1) -> NDArray: ...
def corner_foerstner(image, sigma: float = 1) -> tuple[NDArray, NDArray]: ...
def corner_fast(image, n: int = 12, threshold: float = 0.15) -> NDArray: ...
def corner_subpix(image, corners, window_size: int = 11, alpha: float = 0.99): ...
def corner_peaks(
    image,
    min_distance: int = 1,
    threshold_abs=None,
    threshold_rel=None,
    exclude_border=True,
    indices=True,
    num_peaks=...,
    footprint=None,
    labels=None,
    *,
    num_peaks_per_label=...,
    p_norm: float = ...
) -> np.ndarray: ...
def corner_moravec(image, window_size: int = 1) -> NDArray: ...
def corner_orientations(image, corners, mask): ...



__version__: str = ...

submodules: list = ...

from ._shared.version_requirements import ensure_python_version as ensure_python_version

from ._shared import lazy as lazy

__getattr__, __lazy_dir__, _ = ...

def __dir__(): ...

# Logic for checking for improper install and importing while in the source
# tree when package has not been installed inplace.
# Code adapted from scikit-learn's __check_build module.
_INPLACE_MSG: str = ...

_STANDARD_MSG: str = ...

def _raise_build_error(e): ...

from skimage._shared.tester import PytestTester as PytestTester  # noqa

test = ...



"""
This type stub file was generated by pyright.
"""

from pathlib import Path
from .basic import Booster, Dataset, Sequence, register_logger
from .callback import early_stopping, log_evaluation, print_evaluation, record_evaluation, reset_parameter
from .engine import CVBooster, cv, train
from .sklearn import LGBMClassifier, LGBMModel, LGBMRanker, LGBMRegressor
from .plotting import create_tree_digraph, plot_importance, plot_metric, plot_split_value_histogram, plot_tree
from .dask import DaskLGBMClassifier, DaskLGBMRanker, DaskLGBMRegressor

"""LightGBM, Light Gradient Boosting Machine.

Contributors: https://github.com/microsoft/LightGBM/graphs/contributors.
"""
_version_path = ...
if _version_path.is_file():
    __version__ = ...
__all__ = ['Dataset', 'Booster', 'CVBooster', 'Sequence', 'register_logger', 'train', 'cv', 'LGBMModel', 'LGBMRegressor', 'LGBMClassifier', 'LGBMRanker', 'DaskLGBMRegressor', 'DaskLGBMClassifier', 'DaskLGBMRanker', 'log_evaluation', 'print_evaluation', 'record_evaluation', 'reset_parameter', 'early_stopping', 'plot_importance', 'plot_split_value_histogram', 'plot_metric', 'plot_tree', 'create_tree_digraph']



from typing import Mapping
import itertools
from operator import itemgetter
from ...classes.graph import Graph

__all__ = [
    "local_node_connectivity",
    "node_connectivity",
    "all_pairs_node_connectivity",
]

def local_node_connectivity(G: Graph, source, target, cutoff=None): ...
def node_connectivity(G: Graph, s=None, t=None): ...
def all_pairs_node_connectivity(G: Graph, nbunch=None, cutoff=None) -> Mapping: ...



from numpy.typing import ArrayLike

from collections import defaultdict

from ...classes.graph import Graph
from .quality import modularity
from ...utils import not_implemented_for
from ...utils.mapped_queue import MappedQueue

__all__ = [
    "greedy_modularity_communities",
    "naive_greedy_modularity_communities",
    "_naive_greedy_modularity_communities",
]

def greedy_modularity_communities(
    G: Graph, weight=None, resolution=1, cutoff=1, best_n=None, n_communities=None
) -> ArrayLike: ...
def naive_greedy_modularity_communities(
    G: Graph, resolution=1, weight=None
) -> ArrayLike: ...

# old name
_naive_greedy_modularity_communities = ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.display import DisplayObject, TextDisplayObject
from typing import Iterable, Optional

"""Various display related classes.

Authors : MinRK, gregcaporaso, dannystaple
"""
__all__ = ['Audio', 'IFrame', 'YouTubeVideo', 'VimeoVideo', 'ScribdDocument', 'FileLink', 'FileLinks', 'Code']
class Audio(DisplayObject):
    """Create an audio object.

    When this object is returned by an input cell or passed to the
    display function, it will result in Audio controls being displayed
    in the frontend (only works in the notebook).

    Parameters
    ----------
    data : numpy array, list, unicode, str or bytes
        Can be one of

          * Numpy 1d array containing the desired waveform (mono)
          * Numpy 2d array containing waveforms for each channel.
            Shape=(NCHAN, NSAMPLES). For the standard channel order, see
            http://msdn.microsoft.com/en-us/library/windows/hardware/dn653308(v=vs.85).aspx
          * List of float or integer representing the waveform (mono)
          * String containing the filename
          * Bytestring containing raw PCM data or
          * URL pointing to a file on the web.

        If the array option is used, the waveform will be normalized.

        If a filename or url is used, the format support will be browser
        dependent.
    url : unicode
        A URL to download the data from.
    filename : unicode
        Path to a local file to load the data from.
    embed : boolean
        Should the audio data be embedded using a data URI (True) or should
        the original source be referenced. Set this to True if you want the
        audio to playable later with no internet connection in the notebook.

        Default is `True`, unless the keyword argument `url` is set, then
        default value is `False`.
    rate : integer
        The sampling rate of the raw data.
        Only required when data parameter is being used as an array
    autoplay : bool
        Set to True if the audio should immediately start playing.
        Default is `False`.
    normalize : bool
        Whether audio should be normalized (rescaled) to the maximum possible
        range. Default is `True`. When set to `False`, `data` must be between
        -1 and 1 (inclusive), otherwise an error is raised.
        Applies only when `data` is a list or array of samples; other types of
        audio are never normalized.

    Examples
    --------

    >>> import pytest
    >>> np = pytest.importorskip("numpy")

    Generate a sound

    >>> import numpy as np
    >>> framerate = 44100
    >>> t = np.linspace(0,5,framerate*5)
    >>> data = np.sin(2*np.pi*220*t) + np.sin(2*np.pi*224*t)
    >>> Audio(data, rate=framerate)
    <IPython.lib.display.Audio object>

    Can also do stereo or more channels

    >>> dataleft = np.sin(2*np.pi*220*t)
    >>> dataright = np.sin(2*np.pi*224*t)
    >>> Audio([dataleft, dataright], rate=framerate)
    <IPython.lib.display.Audio object>

    From URL:

    >>> Audio("http://www.nch.com.au/acm/8k16bitpcm.wav")  # doctest: +SKIP
    >>> Audio(url="http://www.w3schools.com/html/horse.ogg")  # doctest: +SKIP

    From a File:

    >>> Audio('IPython/lib/tests/test.wav')  # doctest: +SKIP
    >>> Audio(filename='IPython/lib/tests/test.wav')  # doctest: +SKIP

    From Bytes:

    >>> Audio(b'RAW_WAV_DATA..')  # doctest: +SKIP
    >>> Audio(data=b'RAW_WAV_DATA..')  # doctest: +SKIP

    See Also
    --------
    ipywidgets.Audio

         Audio widget with more more flexibility and options.

    """
    _read_flags = ...
    def __init__(self, data=..., filename=..., url=..., embed=..., rate=..., autoplay=..., normalize=..., *, element_id=...) -> None:
        ...
    
    def reload(self): # -> None:
        """Reload the raw data from file or URL."""
        ...
    
    def src_attr(self):
        ...
    
    def autoplay_attr(self):
        ...
    
    def element_id_attr(self):
        ...
    


class IFrame:
    """
    Generic class to embed an iframe in an IPython notebook
    """
    iframe = ...
    def __init__(self, src, width, height, extras: Optional[Iterable[str]] = ..., **kwargs) -> None:
        ...
    


class YouTubeVideo(IFrame):
    """Class for embedding a YouTube Video in an IPython session, based on its video id.

    e.g. to embed the video from https://www.youtube.com/watch?v=foo , you would
    do::

        vid = YouTubeVideo("foo")
        display(vid)

    To start from 30 seconds::

        vid = YouTubeVideo("abc", start=30)
        display(vid)

    To calculate seconds from time as hours, minutes, seconds use
    :class:`datetime.timedelta`::

        start=int(timedelta(hours=1, minutes=46, seconds=40).total_seconds())

    Other parameters can be provided as documented at
    https://developers.google.com/youtube/player_parameters#Parameters
    
    When converting the notebook using nbconvert, a jpeg representation of the video
    will be inserted in the document.
    """
    def __init__(self, id, width=..., height=..., allow_autoplay=..., **kwargs) -> None:
        ...
    


class VimeoVideo(IFrame):
    """
    Class for embedding a Vimeo video in an IPython session, based on its video id.
    """
    def __init__(self, id, width=..., height=..., **kwargs) -> None:
        ...
    


class ScribdDocument(IFrame):
    """
    Class for embedding a Scribd document in an IPython session

    Use the start_page params to specify a starting point in the document
    Use the view_mode params to specify display type one off scroll | slideshow | book

    e.g to Display Wes' foundational paper about PANDAS in book mode from page 3

    ScribdDocument(71048089, width=800, height=400, start_page=3, view_mode="book")
    """
    def __init__(self, id, width=..., height=..., **kwargs) -> None:
        ...
    


class FileLink:
    """Class for embedding a local file link in an IPython session, based on path

    e.g. to embed a link that was generated in the IPython notebook as my/data.txt

    you would do::

        local_file = FileLink("my/data.txt")
        display(local_file)

    or in the HTML notebook, just::

        FileLink("my/data.txt")
    """
    html_link_str = ...
    def __init__(self, path, url_prefix=..., result_html_prefix=..., result_html_suffix=...) -> None:
        """
        Parameters
        ----------
        path : str
            path to the file or directory that should be formatted
        url_prefix : str
            prefix to be prepended to all files to form a working link [default:
            '']
        result_html_prefix : str
            text to append to beginning to link [default: '']
        result_html_suffix : str
            text to append at the end of link [default: '<br>']
        """
        ...
    
    def __repr__(self):
        """return absolute path to file
        """
        ...
    


class FileLinks(FileLink):
    """Class for embedding local file links in an IPython session, based on path

    e.g. to embed links to files that were generated in the IPython notebook
    under ``my/data``, you would do::

        local_files = FileLinks("my/data")
        display(local_files)

    or in the HTML notebook, just::

        FileLinks("my/data")
    """
    def __init__(self, path, url_prefix=..., included_suffixes=..., result_html_prefix=..., result_html_suffix=..., notebook_display_formatter=..., terminal_display_formatter=..., recursive=...) -> None:
        """
        See :class:`FileLink` for the ``path``, ``url_prefix``,
        ``result_html_prefix`` and ``result_html_suffix`` parameters.

        included_suffixes : list
          Filename suffixes to include when formatting output [default: include
          all files]

        notebook_display_formatter : function
          Used to format links for display in the notebook. See discussion of
          formatter functions below.

        terminal_display_formatter : function
          Used to format links for display in the terminal. See discussion of
          formatter functions below.

        Formatter functions must be of the form::

            f(dirname, fnames, included_suffixes)

        dirname : str
          The name of a directory
        fnames : list
          The files in that directory
        included_suffixes : list
          The file suffixes that should be included in the output (passing None
          meansto include all suffixes in the output in the built-in formatters)
        recursive : boolean
          Whether to recurse into subdirectories. Default is True.

        The function should return a list of lines that will be printed in the
        notebook (if passing notebook_display_formatter) or the terminal (if
        passing terminal_display_formatter). This function is iterated over for
        each directory in self.path. Default formatters are in place, can be
        passed here to support alternative formatting.

        """
        ...
    
    def __repr__(self):
        """return newline-separated absolute paths
        """
        ...
    


class Code(TextDisplayObject):
    """Display syntax-highlighted source code.

    This uses Pygments to highlight the code for HTML and Latex output.

    Parameters
    ----------
    data : str
        The code as a string
    url : str
        A URL to fetch the code from
    filename : str
        A local filename to load the code from
    language : str
        The short name of a Pygments lexer to use for highlighting.
        If not specified, it will guess the lexer based on the filename
        or the code. Available lexers: http://pygments.org/docs/lexers/
    """
    def __init__(self, data=..., url=..., filename=..., language=...) -> None:
        ...
    
    def __repr__(self): # -> None:
        ...
    





from tkinter.messagebox import NO
from matplotlib.colors import Colormap, Normalize
from matplotlib.lines import Line2D
from matplotlib.patches import Patch
from matplotlib.path import Path
import numpy as np
from typing import Callable, Literal, Sequence
from matplotlib._typing import *
from matplotlib.transforms import Bbox, Transform
from matplotlib.cm import ScalarMappable
from matplotlib.backend_bases import Event, MouseEvent
from matplotlib._enums import CapStyle, JoinStyle
from matplotlib.artist import Artist, allow_rasterization

class Collection(Artist, ScalarMappable):
    _transforms: np.ndarray = ...

    def __init__(
        self,
        edgecolors: Color|Sequence[Color]|None=None,
        facecolors: Color|Sequence[Color]|None=None,
        linewidths: float|Sequence[float]|None=None,
        linestyles: str|Sequence[str]|None = None,
        capstyle: CapStyle|None=None,
        joinstyle: JoinStyle|None = None,
        antialiaseds: bool|Sequence[bool]|None=None,
        offsets: Sequence[float]|Sequence[Sequence[float]]|None=None,
        offset_transform: Transform|None=None,
        norm:Normalize|None=None,
        cmap: Colormap|None=None,
        pickradius: float=0.5,
        hatch: str|None=None,
        urls: Sequence[str]|None=None,
        *,
        zorder: int=1,
        **kwargs
    ) -> None: ...
    def get_paths(self): ...
    def set_paths(self)-> None: ...
    def get_transforms(self): ...
    def get_offset_transform(self)-> Transform: ...
    def set_offset_transform(self, offset_transform: Transform)-> None: ...
    def get_datalim(self, transData)-> Bbox: ...
    def get_window_extent(self, renderer=...): ...
    @allow_rasterization
    def draw(self, renderer)-> None: ...
    def set_pickradius(self, pr: float)-> None: ...
    def get_pickradius(self)-> float: ...
    def contains(self, mouseevent: Event)-> tuple[bool, dict]: ...
    def set_urls(self, urls: list[str] | None)-> None: ...
    def get_urls(self)-> list[str]|None: ...
    def set_hatch(
        self, hatch: Literal["/", "\\", "|", "-", "+", "x", "o", "O", ".", "*"]
    )-> None: ...
    def get_hatch(self)-> str: ...
    def set_offsets(self, offsets: ArrayLike)-> None: ...
    def get_offsets(self) -> ArrayLike: ...
    def set_linewidth(self, lw: float | list[float])-> None: ...
    def set_linestyle(self, ls: str | tuple)-> None: ...
    def set_capstyle(self, cs: CapStyle | Literal["butt", "projecting", "round"])-> None: ...
    def get_capstyle(self)-> CapStyle: ...
    def set_joinstyle(self, js: JoinStyle | Literal["miter", "round", "bevel"])-> None: ...
    def get_joinstyle(self)-> JoinStyle: ...
    def set_antialiased(self, aa: bool | list[bool])-> None: ...
    def set_color(self, c: Color | list)-> None: ...
    def set_facecolor(self, c: Color | list[Color])-> None: ...
    def set_facecolors(self, c: Color | list[Color])-> None: ...
    def get_facecolor(self) -> Color | list[Color]: ...
    def get_edgecolor(self) -> Color | list[Color]: ...
    def get_facecolors(self) -> Color | list[Color]: ...
    def get_edgecolors(self) -> Color | list[Color]: ...
    def set_edgecolor(self, c: Color | list[Color] | Literal["face"])-> None: ...
    def set_edgecolors(self, c: Color | list[Color] | Literal["face"])-> None: ...
    def set_alpha(self, alpha: ArrayLike | Scalar | None)-> None: ...
    def get_linewidth(self)-> float: ...
    def get_linestyle(self)-> list[tuple]: ...
    def update_scalarmappable(self)-> None: ...
    def get_fill(self)-> bool: ...
    def update_from(self, other: Collection)-> None: ...

class _CollectionWithSizes(Collection):
    def get_sizes(self) -> np.ndarray: ...
    def set_sizes(self, sizes: np.ndarray | None, dpi: float = ...)-> None: ...
    @allow_rasterization
    def draw(self, renderer)-> None: ...

class PathCollection(_CollectionWithSizes):
    def __init__(self, paths: Sequence[Path], sizes: ArrayLike=..., **kwargs) -> None: ...
    def set_paths(self, paths: Sequence[Path])-> None: ...
    def get_paths(self): ...
    def legend_elements(
        self,
        prop: Literal["colors", "sizes"] = ...,
        num=...,
        fmt=...,
        func: Callable = ...,
        **kwargs
    )-> tuple[list[Line2D], list[str]]: ...

class PolyCollection(_CollectionWithSizes):
    def __init__(self, verts, sizes=..., closed=..., **kwargs) -> None: ...
    def set_verts(self, verts: Sequence[ArrayLike], closed: bool = ...)-> None: ...
    def set_paths(self, verts: Sequence[ArrayLike], closed: bool = ...)-> None: ...
    def set_verts_and_codes(self, verts: Sequence[ArrayLike], codes: ArrayLike)-> None: ...

class BrokenBarHCollection(PolyCollection):
    def __init__(self, xranges: Sequence[Sequence[float]], yrange: Sequence[float], **kwargs) -> None: ...
    @classmethod
    def span_where(cls, x, ymin, ymax, where, **kwargs)->BrokenBarHCollection: ...

class RegularPolyCollection(_CollectionWithSizes):
    def __init__(self, numsides: int, rotation: float=..., sizes: Sequence[float]=..., **kwargs) -> None: ...
    def get_numsides(self)-> int: ...
    def get_rotation(self)-> float: ...
    @allow_rasterization
    def draw(self, renderer)-> None: ...

class StarPolygonCollection(RegularPolyCollection): ...
class AsteriskPolygonCollection(RegularPolyCollection): ...

class LineCollection(Collection):
    def __init__(self, segments: Sequence[ArrayLike], *, zorder: int=2, **kwargs) -> None: ...
    def set_segments(self, segments:Sequence[ArrayLike])-> None: ...
    def set_verts(self, segments:Sequence[ArrayLike])-> None: ...
    def set_paths(self, segments:Sequence[ArrayLike])-> None: ...

    def get_segments(self) -> list: ...
    def set_color(self, c: Color | list[Color]): ...
    def set_colors(self, c: Color | list[Color]): ...
    def get_color(self)-> Color: ...
    def get_colors(self)-> Color: ...


class EventCollection(LineCollection):
    def __init__(
        self,
        positions,
        orientation: str='horizontal',
        lineoffset: float=0,
        linelength: float=1,
        linewidth:float|Sequence[float]|None=None,
        color: Color|Sequence[Color]|None = None,
        linestyle:str|Sequence[str]='solid',
        antialiased: bool|Sequence[bool]|None=None,
        **kwargs
    ) -> None: ...
    def get_positions(self)-> list: ...
    def set_positions(self, positions: ArrayLike)-> None: ...
    def add_positions(self, position: ArrayLike)-> None: ...
    def extend_positions(self, position: ArrayLike)-> None: ...
    def append_positions(self, position: ArrayLike)-> None: ...
    def is_horizontal(self)-> bool: ...
    def get_orientation(self)-> str: ...
    def switch_orientation(self)-> None: ...
    def set_orientation(self, orientation: Literal["horizontal", "vertical"])-> None: ...
    def get_linelength(self)-> float: ...
    def set_linelength(self, linelength: float)-> None: ...
    def get_lineoffset(self)-> float: ...
    def set_lineoffset(self, lineoffset: float)-> None: ...
    def get_linewidth(self)-> float: ...
    def get_linewidths(self)-> list[float]: ...
    def get_color(self)-> Color: ...

class CircleCollection(_CollectionWithSizes):
    def __init__(self, sizes, **kwargs) -> None: ...

class EllipseCollection(Collection):
    def __init__(self, widths: ArrayLike, heights: ArrayLike, angles: ArrayLike, units:str=..., **kwargs) -> None: ...
    @allow_rasterization
    def draw(self, renderer)-> None: ...

class PatchCollection(Collection):
    def __init__(self, patches: Sequence[Patch], match_original: bool=..., **kwargs) -> None: ...
    def set_paths(self, patches)-> None: ...

class TriMesh(Collection):
    def __init__(self, triangulation, **kwargs) -> None: ...
    def get_paths(self)-> list[Path]: ...
    def set_paths(self)-> None: ...
    @staticmethod
    def convert_mesh_to_paths(tri)-> list[Path]: ...
    @allow_rasterization
    def draw(self, renderer)-> None: ...

class QuadMesh(Collection):
    def __init__(self, *args, **kwargs) -> None: ...
    def get_paths(self)-> list[Path]: ...
    def set_paths(self)-> None: ...
    def set_array(self, A: ArrayLike)-> None: ...
    def get_datalim(self, transData): ...
    def get_coordinates(self)-> np.ndarray: ...
    @staticmethod
    def convert_mesh_to_paths(meshWidth, meshHeight, coordinates)-> list[Path]: ...
    @allow_rasterization
    def draw(self, renderer)-> None: ...
    def get_cursor_data(self, event: MouseEvent): ...



from typing import Callable
from scipy.sparse.linalg import LinearOperator
from numpy import block as bmat, ndarray
from scipy.linalg import (
    inv as inv,
    eigh as eigh,
    cho_factor as cho_factor,
    cho_solve as cho_solve,
    cholesky as cholesky,
    LinAlgError as LinAlgError,
)
from scipy.sparse import isspmatrix as isspmatrix
from .._typing import MatrixLike, ArrayLike, Float, Scalar, Int
import inspect
import warnings
import numpy as np

__all__ = ["lobpcg"]


def lobpcg(
    A: MatrixLike | LinearOperator | ArrayLike | Callable,
    X: ArrayLike | Float,
    B: ArrayLike | None | Callable | MatrixLike | LinearOperator = None,
    M: ArrayLike | None | Callable | MatrixLike | LinearOperator = None,
    Y: float | None | ArrayLike = None,
    tol: None | Scalar = None,
    maxiter: None | int = None,
    largest: None | bool = True,
    verbosityLevel: Int = 0,
    retLambdaHistory: None | bool = False,
    retResidualNormsHistory: None | bool = False,
    restartControl: None | int = 20,
) -> tuple[ndarray, ndarray, ndarray, ndarray]:
    ...



from typing import Mapping
import functools
from ...classes.graph import Graph

from ...exception import NetworkXError
from ...utils.decorators import not_implemented_for

__all__ = ["closeness_centrality", "incremental_closeness_centrality"]

def closeness_centrality(
    G: Graph, u=None, distance=None, wf_improved=True
) -> Mapping: ...
def incremental_closeness_centrality(
    G: Graph,
    edge: tuple,
    prev_cc: Mapping | None = None,
    insertion: bool = True,
    wf_improved=True,
) -> Mapping: ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from __future__ import print_function

import numpy as np
import sys
import os
import inspect
import gc

import functools

from distutils.version import LooseVersion

from ..util.check_environment import has_backend

skipif = ...

IS_TRAVIS_CI = ...
IS_GITHUB_ACTIONS = ...
IS_CI = ...

def SkipTest(*args, **kwargs): ...
def _safe_rep(obj, short=False): ...
def _safe_str(obj): ...
def _format_msg(msg, std_msg): ...
def nottest(func): ...
def assert_raises(exp, func, *args, **kwargs): ...
def assert_in(member, container, msg=None): ...
def assert_true(x, msg=None): ...
def assert_equal(x, y, msg=None): ...
def assert_not_equal(x, y, msg=None): ...
def assert_not_in(member, container, msg=None): ...
def assert_is(expr1, expr2, msg=None): ...

class raises(object):
    def __init__(self, exc): ...
    def __enter__(self): ...
    def __exit__(self, exc_typ, exc, tb): ...

###############################################################################
# GL stuff

def has_pyopengl(): ...
def requires_pyopengl(): ...
def requires_ssl(): ...

###############################################################################
# App stuff

def has_application(backend=None, has=..., capable=...): ...
def composed(*decs): ...
def garbage_collect(f): ...
def requires_application(backend=None, has=..., capable=..., force_gc=True): ...
def requires_img_lib(): ...
def has_ipython(version="3.0"): ...
def requires_ipython(version="3.0"): ...
def requires_numpydoc(): ...

###############################################################################
# Visuals stuff

def _has_scipy(min_version): ...
def requires_scipy(min_version="0.13"): ...
def _bad_glfw_decorate(app): ...
@nottest
def TestingCanvas(bgcolor="black", size=..., dpi=None, decorate=None, **kwargs): ...
@nottest
def save_testing_image(image, location): ...
@nottest
def run_tests_if_main(): ...
def run_tests_in_object(ob): ...



from .basic import (
    is_bipartite as is_bipartite,
    is_bipartite_node_set as is_bipartite_node_set,
    color as color,
    sets as sets,
    density as density,
    degrees as degrees,
)
from .centrality import (
    degree_centrality as degree_centrality,
    betweenness_centrality as betweenness_centrality,
    closeness_centrality as closeness_centrality,
)
from .cluster import (
    clustering as clustering,
    average_clustering as average_clustering,
    latapy_clustering as latapy_clustering,
    robins_alexander_clustering as robins_alexander_clustering,
)
from .covering import min_edge_cover as min_edge_cover
from .edgelist import (
    generate_edgelist as generate_edgelist,
    write_edgelist as write_edgelist,
    parse_edgelist as parse_edgelist,
    read_edgelist as read_edgelist,
)
from .matching import (
    maximum_matching as maximum_matching,
    hopcroft_karp_matching as hopcroft_karp_matching,
    eppstein_matching as eppstein_matching,
    to_vertex_cover as to_vertex_cover,
    minimum_weight_full_matching as minimum_weight_full_matching,
)
from .matrix import (
    biadjacency_matrix as biadjacency_matrix,
    from_biadjacency_matrix as from_biadjacency_matrix,
)
from .projection import (
    project as project,
    projected_graph as projected_graph,
    weighted_projected_graph as weighted_projected_graph,
    collaboration_weighted_projected_graph as collaboration_weighted_projected_graph,
    overlap_weighted_projected_graph as overlap_weighted_projected_graph,
    generic_weighted_projected_graph as generic_weighted_projected_graph,
)

from .redundancy import node_redundancy as node_redundancy
from .spectral import spectral_bipartivity as spectral_bipartivity
from .generators import (
    configuration_model as configuration_model,
    havel_hakimi_graph as havel_hakimi_graph,
    reverse_havel_hakimi_graph as reverse_havel_hakimi_graph,
    alternating_havel_hakimi_graph as alternating_havel_hakimi_graph,
    preferential_attachment_graph as preferential_attachment_graph,
    random_graph as random_graph,
    gnmk_random_graph as gnmk_random_graph,
    complete_bipartite_graph as complete_bipartite_graph,
)



"""
This type stub file was generated by pyright.
"""

import datatable
from pandas import concat
from pandas.api.types import is_sparse as is_dtype_sparse
from dask import delayed
from dask.array import from_delayed as dask_array_from_delayed
from dask.bag import from_delayed as dask_bag_from_delayed
from dask.distributed import default_client, wait

"""Compatibility library."""
PANDAS_INSTALLED = ...
PANDAS_INSTALLED = ...
class pd_Series:
    """Dummy class for pandas.Series."""
    ...


class pd_DataFrame:
    """Dummy class for pandas.DataFrame."""
    ...


concat = ...
is_dtype_sparse = ...
MATPLOTLIB_INSTALLED = ...
MATPLOTLIB_INSTALLED = ...
GRAPHVIZ_INSTALLED = ...
GRAPHVIZ_INSTALLED = ...
if hasattr(datatable, "Frame"):
    dt_DataTable = ...
else:
    dt_DataTable = ...
DATATABLE_INSTALLED = ...
DATATABLE_INSTALLED = ...
class dt_DataTable:
    """Dummy class for datatable.DataTable."""
    ...


SKLEARN_INSTALLED = ...
_LGBMModelBase = ...
_LGBMRegressorBase = ...
_LGBMClassifierBase = ...
_LGBMLabelEncoder = ...
LGBMNotFittedError = ...
_LGBMStratifiedKFold = ...
_LGBMGroupKFold = ...
_LGBMCheckXY = ...
_LGBMCheckArray = ...
_LGBMCheckSampleWeight = ...
_LGBMAssertAllFinite = ...
_LGBMCheckClassificationTargets = ...
_LGBMComputeSampleWeight = ...
SKLEARN_INSTALLED = ...
class _LGBMModelBase:
    """Dummy class for sklearn.base.BaseEstimator."""
    ...


class _LGBMClassifierBase:
    """Dummy class for sklearn.base.ClassifierMixin."""
    ...


class _LGBMRegressorBase:
    """Dummy class for sklearn.base.RegressorMixin."""
    ...


_LGBMLabelEncoder = ...
LGBMNotFittedError = ...
_LGBMStratifiedKFold = ...
_LGBMGroupKFold = ...
_LGBMCheckXY = ...
_LGBMCheckArray = ...
_LGBMCheckSampleWeight = ...
_LGBMAssertAllFinite = ...
_LGBMCheckClassificationTargets = ...
_LGBMComputeSampleWeight = ...
DASK_INSTALLED = ...
DASK_INSTALLED = ...
dask_array_from_delayed = ...
dask_bag_from_delayed = ...
delayed = ...
default_client = ...
wait = ...
class Client:
    """Dummy class for dask.distributed.Client."""
    ...


class dask_Array:
    """Dummy class for dask.array.Array."""
    ...


class dask_DataFrame:
    """Dummy class for dask.dataframe.DataFrame."""
    ...


class dask_Series:
    """Dummy class for dask.dataframe.Series."""
    ...





from typing import Mapping
from numpy.typing import ArrayLike
from numbers import Number

from ..classes.graph import Graph
from ..drawing.layout import (
    circular_layout,
    kamada_kawai_layout,
    planar_layout,
    random_layout,
    shell_layout,
    spectral_layout,
    spring_layout,
)

__all__ = [
    "draw",
    "draw_networkx",
    "draw_networkx_nodes",
    "draw_networkx_edges",
    "draw_networkx_labels",
    "draw_networkx_edge_labels",
    "draw_circular",
    "draw_kamada_kawai",
    "draw_random",
    "draw_spectral",
    "draw_spring",
    "draw_planar",
    "draw_shell",
]

def draw(G: Graph, pos: Mapping | None = None, ax=None, **kwds): ...
def draw_networkx(
    G: Graph, pos: Mapping | None = None, arrows=None, with_labels=True, **kwds
): ...
def draw_networkx_nodes(
    G: Graph,
    pos: Mapping,
    nodelist=None,
    node_size=300,
    node_color="#1f78b4",
    node_shape="o",
    alpha=None,
    cmap=None,
    vmin=None,
    vmax=None,
    ax=None,
    linewidths=None,
    edgecolors=None,
    label=None,
    margins=None,
): ...
def draw_networkx_edges(
    G: Graph,
    pos: Mapping,
    edgelist=None,
    width=1.0,
    edge_color="k",
    style="solid",
    alpha=None,
    arrowstyle=None,
    arrowsize=10,
    edge_cmap=None,
    edge_vmin: float | None = None,
    edge_vmax: float | None = None,
    ax=None,
    arrows=None,
    label: str | None = None,
    node_size=300,
    nodelist=None,
    node_shape="o",
    connectionstyle="arc3",
    min_source_margin=0,
    min_target_margin=0,
): ...
def draw_networkx_labels(
    G: Graph,
    pos: Mapping,
    labels=None,
    font_size=12,
    font_color="k",
    font_family="sans-serif",
    font_weight="normal",
    alpha=None,
    bbox=None,
    horizontalalignment="center",
    verticalalignment="center",
    ax=None,
    clip_on=True,
) -> Mapping: ...
def draw_networkx_edge_labels(
    G: Graph,
    pos: Mapping,
    edge_labels=None,
    label_pos=0.5,
    font_size=10,
    font_color="k",
    font_family="sans-serif",
    font_weight="normal",
    alpha=None,
    bbox=None,
    horizontalalignment="center",
    verticalalignment="center",
    ax=None,
    rotate=True,
    clip_on=True,
) -> Mapping: ...
def draw_circular(G: Graph, **kwargs): ...
def draw_kamada_kawai(G: Graph, **kwargs): ...
def draw_random(G: Graph, **kwargs): ...
def draw_spectral(G: Graph, **kwargs): ...
def draw_spring(G: Graph, **kwargs): ...
def draw_shell(G: Graph, nlist=None, **kwargs): ...
def draw_planar(G: Graph, **kwargs): ...
def apply_alpha(
    colors,
    alpha: float | ArrayLike,
    elem_list,
    cmap=None,
    vmin: float | None = None,
    vmax: float | None = None,
): ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import base64
import logging
import sys
import inspect
import re
import traceback
import json
from functools import partial

import numpy as np
from numpy.typing import ArrayLike

###############################################################################
# LOGGING (some adapted from mne-python)

def _get_vispy_caller(): ...

# class _WrapStdOut(object):
#     """Class to work around how doctest captures stdout"""
#     def __getattr__(self, name):
#         # Even more ridiculous than this class, this must be sys.stdout (not
#         # just stdout) in order for this to work (tested on OSX and Linux)
#         return getattr(sys.stdout, name)

class _VispyFormatter(logging.Formatter):
    def __init__(self): ...
    def _vispy_set_prepend(self, prepend): ...
    def format(self, record): ...

class _VispyStreamHandler(logging.StreamHandler):
    def __init__(self): ...
    def _vispy_emit_match_andor_record(self, record): ...
    def _vispy_set_match(self, match): ...
    def _vispy_set_emit_record(self, record): ...
    def _vispy_reset_list(self): ...

logger = ...
_lf = ...
_lh = ...  # needs _lf to exist

logging_types = ...

def set_log_level(verbose: bool | str | int | None, match: str | None = None, return_old: bool = False): ...

class use_log_level(object):

    # This method mostly wraps to set_log_level, but also takes
    # care of enabling/disabling message recording in the formatter.

    def __init__(
        self,
        level: str,
        match: str | None = None,
        record: bool = False,
        print_msg: bool = True,
    ): ...
    def __enter__(self): ...
    def __exit__(self, type, value, traceback): ...

def log_exception(level: str = "warning", tb_skip: int = 2): ...

logger.log_exception = ...  # make this easier to reach

def _handle_exception(ignore_callback_errors, print_callback_errors, obj, cb_event=None, node=None): ...
def _serialize_buffer(buffer, array_serialization=None): ...

class NumPyJSONEncoder(json.JSONEncoder):
    def default(self, obj): ...



from datetime import datetime, timedelta
from typing import (
    Any,
    Callable,
    Iterable,
    List,
    Optional,
    Pattern,
    Sequence,
    Tuple,
    Type,
    Union,
)

from django.core.validators import BaseValidator
from django.forms.boundfield import BoundField
from django.forms.forms import BaseForm
from django.forms.widgets import Widget

_Choice = Tuple[Any, str]
_ChoiceNamedGroup = Tuple[str, Iterable[_Choice]]
_FieldChoices = Iterable[Union[_Choice, _ChoiceNamedGroup]]

class Field:
    initial: Any
    label: Optional[str]
    required: bool
    widget: Union[Type[Widget], Widget] = ...
    hidden_widget: Any = ...
    default_validators: Any = ...
    default_error_messages: Any = ...
    empty_values: Any = ...
    show_hidden_initial: bool = ...
    help_text: str = ...
    disabled: bool = ...
    label_suffix: Optional[Any] = ...
    localize: bool = ...
    error_messages: Any = ...
    validators: List[BaseValidator] = ...
    max_length: Optional[Union[int, str]] = ...
    choices: _FieldChoices = ...
    base_field: Field
    def __init__(
        self,
        *,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...
    def prepare_value(self, value: Any) -> Any: ...
    def to_python(self, value: Optional[Any]) -> Optional[Any]: ...
    def validate(self, value: Any) -> None: ...
    def run_validators(self, value: Any) -> None: ...
    def clean(self, value: Any) -> Any: ...
    def bound_data(self, data: Any, initial: Any) -> Any: ...
    def widget_attrs(self, widget: Widget) -> Any: ...
    def has_changed(self, initial: Any, data: Any) -> bool: ...
    def get_bound_field(self, form: BaseForm, field_name: str) -> BoundField: ...
    def deconstruct(self) -> Any: ...

class CharField(Field):
    min_length: Optional[Union[int, str]] = ...
    strip: bool = ...
    empty_value: Optional[str] = ...
    def __init__(
        self,
        max_length: Optional[Any] = ...,
        min_length: Optional[Any] = ...,
        strip: bool = ...,
        empty_value: Optional[str] = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class IntegerField(Field):
    max_value: Optional[Any]
    min_value: Optional[Any]
    re_decimal: Any = ...
    def __init__(
        self,
        max_value: Optional[Any] = ...,
        min_value: Optional[Any] = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class FloatField(IntegerField): ...

class DecimalField(IntegerField):
    decimal_places: Optional[int]
    max_digits: Optional[int]
    def __init__(
        self,
        *,
        max_value: Optional[Any] = ...,
        min_value: Optional[Any] = ...,
        max_digits: Optional[Any] = ...,
        decimal_places: Optional[Any] = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class BaseTemporalField(Field):
    input_formats: Any = ...
    def __init__(
        self,
        input_formats: Optional[Any] = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...
    def strptime(self, value: Any, format: str) -> Any: ...

class DateField(BaseTemporalField): ...
class TimeField(BaseTemporalField): ...
class DateTimeField(BaseTemporalField): ...

class DurationField(Field):
    def prepare_value(
        self, value: Optional[Union[timedelta, str]]
    ) -> Optional[str]: ...

class RegexField(CharField):
    regex: str = ...
    def __init__(
        self,
        regex: Union[str, Pattern[str]],
        max_length: Optional[Any] = ...,
        min_length: Optional[Any] = ...,
        strip: bool = ...,
        empty_value: Optional[str] = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class EmailField(CharField): ...

class FileField(Field):
    allow_empty_file: bool = ...
    def __init__(
        self,
        max_length: Optional[Any] = ...,
        allow_empty_file: bool = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...
    def clean(self, data: Any, initial: Optional[Any] = ...) -> Any: ...

class ImageField(FileField): ...
class URLField(CharField): ...
class BooleanField(Field): ...
class NullBooleanField(BooleanField): ...

class CallableChoiceIterator:
    choices_func: Callable[..., Any] = ...
    def __init__(self, choices_func: Callable[..., Any]) -> None: ...
    def __iter__(self) -> None: ...

class ChoiceField(Field):
    def __init__(
        self,
        choices: Union[_FieldChoices, Callable[[], _FieldChoices]] = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...
    def valid_value(self, value: str) -> bool: ...

class TypedChoiceField(ChoiceField):
    coerce: Union[Callable[..., Any], Type[Any]] = ...
    empty_value: Optional[str] = ...
    def __init__(
        self,
        coerce: Any = ...,
        empty_value: Optional[str] = ...,
        choices: Any = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class MultipleChoiceField(ChoiceField): ...

class TypedMultipleChoiceField(MultipleChoiceField):
    coerce: Union[Callable[..., Any], Type[float]] = ...
    empty_value: Optional[List[Any]] = ...
    def __init__(
        self,
        coerce: Any = ...,
        empty_value: Optional[str] = ...,
        choices: Any = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class ComboField(Field):
    fields: Any = ...
    def __init__(
        self,
        fields: Sequence[Field],
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class MultiValueField(Field):
    require_all_fields: bool = ...
    fields: Any = ...
    def __init__(
        self,
        fields: Sequence[Field],
        require_all_fields: bool = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...
    def compress(self, data_list: Any) -> Any: ...

class FilePathField(ChoiceField):
    allow_files: bool
    allow_folders: bool
    match: Optional[str]
    path: str
    recursive: bool
    match_re: Any = ...
    def __init__(
        self,
        path: str,
        match: Optional[Any] = ...,
        recursive: bool = ...,
        allow_files: bool = ...,
        allow_folders: bool = ...,
        choices: Any = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class SplitDateTimeField(MultiValueField):
    def __init__(
        self,
        input_date_formats: Optional[Any] = ...,
        input_time_formats: Optional[Any] = ...,
        fields: Sequence[Field] = ...,
        require_all_fields: bool = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...
    def compress(self, data_list: List[Optional[datetime]]) -> Optional[datetime]: ...

class GenericIPAddressField(CharField):
    unpack_ipv4: bool = ...
    def __init__(
        self,
        protocol: str = ...,
        unpack_ipv4: bool = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class SlugField(CharField):
    allow_unicode: bool = ...
    def __init__(
        self,
        allow_unicode: bool = ...,
        required: bool = ...,
        widget: Optional[Union[Widget, Type[Widget]]] = ...,
        label: Optional[Any] = ...,
        initial: Optional[Any] = ...,
        help_text: str = ...,
        error_messages: Optional[Any] = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[Any] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: Optional[Any] = ...,
    ) -> None: ...

class UUIDField(CharField): ...
class InvalidJSONInput(str): ...
class JSONString(str): ...

class JSONField(CharField):
    default_error_messages: Any = ...
    widget: Any = ...
    encoder: Any = ...
    decoder: Any = ...
    def __init__(
        self, encoder: Optional[Any] = ..., decoder: Optional[Any] = ..., **kwargs: Any
    ) -> None: ...
    def to_python(self, value: Any) -> Any: ...
    def bound_data(self, data: Any, initial: Any) -> Any: ...
    def prepare_value(self, value: Any) -> Any: ...
    def has_changed(self, initial: Any, data: Any) -> Any: ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, line_magic, magics_class
from IPython.testing.skipdoctest import skip_doctest

"""
%store magic for lightweight persistence.

Stores variables, aliases and macros in IPython's database.

To automatically restore stored variables at startup, add this to your
:file:`ipython_config.py` file::

  c.StoreMagics.autorestore = True
"""
def restore_aliases(ip, alias=...): # -> None:
    ...

def refresh_variables(ip): # -> None:
    ...

def restore_dhist(ip): # -> None:
    ...

def restore_data(ip): # -> None:
    ...

@magics_class
class StoreMagics(Magics):
    """Lightweight persistence for python variables.

    Provides the %store magic."""
    autorestore = ...
    def __init__(self, shell) -> None:
        ...
    
    @skip_doctest
    @line_magic
    def store(self, parameter_s=...): # -> None:
        """Lightweight persistence for python variables.

        Example::

          In [1]: l = ['hello',10,'world']
          In [2]: %store l
          Stored 'l' (list)
          In [3]: exit

          (IPython session is closed and started again...)

          ville@badger:~$ ipython
          In [1]: l
          NameError: name 'l' is not defined
          In [2]: %store -r
          In [3]: l
          Out[3]: ['hello', 10, 'world']

        Usage:

        * ``%store``          - Show list of all variables and their current
                                values
        * ``%store spam bar`` - Store the *current* value of the variables spam
                                and bar to disk
        * ``%store -d spam``  - Remove the variable and its value from storage
        * ``%store -z``       - Remove all variables from storage
        * ``%store -r``       - Refresh all variables, aliases and directory history
                                from store (overwrite current vals)
        * ``%store -r spam bar`` - Refresh specified variables and aliases from store
                                   (delete current val)
        * ``%store foo >a.txt``  - Store value of foo to new file a.txt
        * ``%store foo >>a.txt`` - Append value of foo to file a.txt

        It should be noted that if you change the value of a variable, you
        need to %store it again if you want to persist the new value.

        Note also that the variables will need to be pickleable; most basic
        python types can be safely %store'd.

        Also aliases can be %store'd across sessions.
        To remove an alias from the storage, use the %unalias magic.
        """
        ...
    


def load_ipython_extension(ip): # -> None:
    """Load the extension in IPython."""
    ...




from itertools import chain as chain
from numpy import ndarray
from scipy.sparse import (
    issparse as issparse,
    dok_matrix as dok_matrix,
    lil_matrix as lil_matrix,
)
from ..utils._array_api import get_namespace as get_namespace
from .validation import check_array as check_array
from .._typing import ArrayLike, MatrixLike
from collections.abc import Sequence as Sequence

# Author: Arnaud Joly, Joel Nothman, Hamzeh Alsalhi
#
# License: BSD 3 clause
import warnings

import numpy as np


_FN_UNIQUE_LABELS: dict = ...


def unique_labels(*ys) -> ndarray:
    ...


def is_multilabel(y: ArrayLike) -> bool:
    ...


def check_classification_targets(y: ArrayLike) -> None:
    ...


def type_of_target(y: MatrixLike | ArrayLike, input_name: str = "") -> str:
    ...


def class_distribution(
    y: MatrixLike, sample_weight: None | ArrayLike = None
) -> tuple[list[ndarray], list[int], list[ndarray]]:
    ...



from typing import Any, ClassVar, TypeVar
from ..utils import check_random_state as check_random_state
from scipy.special import logsumexp as logsumexp
from abc import ABCMeta, abstractmethod as abstractmethod
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from numpy.random.mtrand import RandomState
from ..base import BaseEstimator, DensityMixin
from ..cluster import kmeans_plusplus as kmeans_plusplus
from time import time as time
from .._typing import MatrixLike, Float, Int
from .. import cluster as cluster
from ..utils.validation import check_is_fitted as check_is_fitted

BaseMixture_Self = TypeVar("BaseMixture_Self", bound="BaseMixture")


# Author: Wei Xue <xuewei4d@gmail.com>
# Modified by Thierry Guillemot <thierry.guillemot.work@gmail.com>
# License: BSD 3 clause

import warnings

import numpy as np


class BaseMixture(DensityMixin, BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: int,
        tol: float,
        reg_covar: float | int,
        max_iter: int,
        n_init: int,
        init_params: str,
        random_state: None | RandomState | int,
        warm_start: bool,
        verbose: int,
        verbose_interval: int,
    ) -> None:
        ...

    def fit(self: BaseMixture_Self, X: MatrixLike, y: Any = None) -> BaseMixture_Self:
        ...

    def fit_predict(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...

    def score_samples(self, X: MatrixLike) -> ndarray:
        ...

    def score(self, X: MatrixLike, y: Any = None) -> Float:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def sample(self, n_samples: Int = 1) -> tuple[ndarray, ndarray]:
        ...



from typing import Callable, ClassVar, Literal, TypeVar
from abc import ABCMeta, abstractmethod as abstractmethod
from scipy import sparse as sparse
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ..neighbors import NearestNeighbors as NearestNeighbors
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from scipy.sparse import csgraph as csgraph
from ..base import BaseEstimator, ClassifierMixin
from ..metrics.pairwise import rbf_kernel as rbf_kernel
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils.validation import check_is_fitted as check_is_fitted

BaseLabelPropagation_Self = TypeVar(
    "BaseLabelPropagation_Self", bound="BaseLabelPropagation"
)
LabelPropagation_Self = TypeVar("LabelPropagation_Self", bound="LabelPropagation")

# coding=utf8

import warnings
import numpy as np


class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        kernel: Callable | Literal["rbf", "knn", "rbf"] = "rbf",
        *,
        gamma: Float = 20,
        n_neighbors: Int = 7,
        alpha: Float = 1,
        max_iter: Int = 30,
        tol: Float = 1e-3,
        n_jobs=None,
    ) -> None:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def fit(
        self: BaseLabelPropagation_Self, X: MatrixLike, y: ArrayLike
    ) -> BaseLabelPropagation_Self | LabelSpreading:
        ...


class LabelPropagation(BaseLabelPropagation):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    transduction_: ndarray = ...
    label_distributions_: ndarray = ...
    classes_: ndarray = ...
    X_: ndarray = ...

    _variant: ClassVar[str] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        kernel: Callable | Literal["knn", "rbf", "rbf"] = "rbf",
        *,
        gamma: Float = 20,
        n_neighbors: Int = 7,
        max_iter: Int = 1000,
        tol: float = 1e-3,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: LabelPropagation_Self, X: MatrixLike, y: ArrayLike
    ) -> LabelPropagation_Self:
        ...


class LabelSpreading(BaseLabelPropagation):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    transduction_: ndarray = ...
    label_distributions_: ndarray = ...
    classes_: ndarray = ...
    X_: ndarray = ...

    _variant: ClassVar[str] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        kernel: Callable | Literal["rbf", "knn", "rbf"] = "rbf",
        *,
        gamma: Float = 20,
        n_neighbors: Int = 7,
        alpha: Float = 0.2,
        max_iter: Int = 30,
        tol: Float = 1e-3,
        n_jobs: None | Int = None,
    ) -> None:
        ...



import numpy as np
from scipy.ndimage import maximum_filter, minimum_filter, convolve

from ..transform import integral_image
from ..feature import structure_tensor
from ..morphology import octagon, star
from ..feature.util import (
    FeatureDetector,
    _prepare_grayscale_input_2D,
    _mask_border_keypoints,
)
from .._shared.utils import check_nD

# The paper(Reference [1]) mentions the sizes of the Octagon shaped filter
# kernel for the first seven scales only. The sizes of the later scales
# have been extrapolated based on the following statement in the paper.
# "These octagons scale linearly and were experimentally chosen to correspond
# to the seven DOBs described in the previous section."
OCTAGON_OUTER_SHAPE: list = ...
OCTAGON_INNER_SHAPE: list = ...

# The sizes for the STAR shaped filter kernel for different scales have been
# taken from the OpenCV implementation.
STAR_SHAPE: list = ...
STAR_FILTER_SHAPE: list = ...

def _filter_image(image, min_scale, max_scale, mode): ...
def _octagon_kernel(mo, no, mi, ni): ...
def _star_kernel(m, n): ...
def _suppress_lines(feature_mask, image, sigma, line_threshold): ...

class CENSURE(FeatureDetector):
    def __init__(
        self,
        min_scale=1,
        max_scale=7,
        mode="DoB",
        non_max_threshold=0.15,
        line_threshold=10,
    ): ...
    def detect(self, image): ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
# Author: Siddharth Bhat
# -----------------------------------------------------------------------------

from functools import lru_cache

import numpy as np

from . import Visual, TextVisual, CompoundVisual, _BorderVisual

# from .border import _BorderVisual
from .shaders import Function
from ..color import get_colormap

_VERTEX_SHADER: str = ...  # noqa

_FRAGMENT_SHADER: str = ...  # noqa

class _CoreColorBarVisual(Visual):

    _shaders: dict = ...

    def __init__(self, pos, halfdim, cmap, orientation, **kwargs): ...
    def _update(self): ...
    @staticmethod
    @lru_cache(maxsize=4)
    def _get_texcoord_func(orientation): ...
    @staticmethod
    def _check_orientation(orientation): ...
    @property
    def pos(self): ...
    @pos.setter
    def pos(self, pos): ...
    @property
    def halfdim(self): ...
    @halfdim.setter
    def halfdim(self, halfdim): ...
    @property
    def cmap(self): ...
    @cmap.setter
    def cmap(self, cmap): ...
    @staticmethod
    def _prepare_transforms(view): ...
    def _prepare_draw(self, view): ...

class ColorBarVisual(CompoundVisual):

    # The padding multiplier that's used to place the text
    # next to the Colorbar. Makes sure the text isn't
    # visually "sticking" to the Colorbar
    text_padding_factor: float = ...

    def __init__(
        self,
        cmap,
        orientation,
        size,
        pos=[0, 0],
        label="",
        label_color="black",
        clim=...,
        border_width=1.0,
        border_color="black",
    ): ...
    def _update(self): ...
    def _update_positions(self): ...
    @staticmethod
    def _get_label_anchors(center, halfdim, orientation, transforms): ...
    @staticmethod
    def _get_ticks_anchors(center, halfdim, orientation, transforms): ...
    @staticmethod
    def _calc_positions(center, halfdim, border_width, orientation, transforms): ...
    @property
    def pos(self): ...
    @pos.setter
    def pos(self, pos): ...
    @property
    def cmap(self): ...
    @cmap.setter
    def cmap(self, cmap): ...
    @property
    def clim(self): ...
    @clim.setter
    def clim(self, clim): ...
    @property
    def label(self): ...
    @label.setter
    def label(self, label): ...
    @property
    def ticks(self): ...
    @ticks.setter
    def ticks(self, ticks): ...
    @property
    def border_width(self): ...
    @border_width.setter
    def border_width(self, border_width): ...
    @property
    def border_color(self): ...
    @border_color.setter
    def border_color(self, border_color): ...
    @property
    def orientation(self): ...
    @property
    def size(self): ...
    @size.setter
    def size(self, size): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.ndimage._nd_image, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def binary_erosion() -> typing.Any:
    ...

def binary_erosion2() -> typing.Any:
    ...

def correlate() -> typing.Any:
    ...

def correlate1d() -> typing.Any:
    ...

def distance_transform_bf() -> typing.Any:
    ...

def distance_transform_op() -> typing.Any:
    ...

def euclidean_feature_transform() -> typing.Any:
    ...

def find_objects() -> typing.Any:
    ...

def fourier_filter() -> typing.Any:
    ...

def fourier_shift() -> typing.Any:
    ...

def generic_filter() -> typing.Any:
    ...

def generic_filter1d() -> typing.Any:
    ...

def geometric_transform() -> typing.Any:
    ...

def min_or_max_filter() -> typing.Any:
    ...

def min_or_max_filter1d() -> typing.Any:
    ...

def rank_filter() -> typing.Any:
    ...

def spline_filter1d() -> typing.Any:
    ...

def uniform_filter1d() -> typing.Any:
    ...

def watershed_ift() -> typing.Any:
    ...

def zoom_shift() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import ClassVar, Literal, TypeVar
from ._newton_solver import NewtonCholeskySolver as NewtonCholeskySolver, NewtonSolver
from .._linear_loss import LinearModelLoss as LinearModelLoss
from ...utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from ..._loss.loss import (
    HalfGammaLoss as HalfGammaLoss,
    HalfPoissonLoss as HalfPoissonLoss,
    HalfSquaredError as HalfSquaredError,
    HalfTweedieLoss as HalfTweedieLoss,
    HalfTweedieLossIdentity as HalfTweedieLossIdentity,
)
from ...utils.validation import check_is_fitted as check_is_fitted
from ..._loss.loss import BaseLoss
from ..._typing import Float, Int, MatrixLike, ArrayLike
from ...utils import check_array as check_array, deprecated
from ..._loss.glm_distribution import TweedieDistribution as TweedieDistribution
from ...base import BaseEstimator, RegressorMixin

_GeneralizedLinearRegressor_Self = TypeVar(
    "_GeneralizedLinearRegressor_Self", bound="_GeneralizedLinearRegressor"
)


import numpy as np
import scipy.optimize


class _GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
    _base_loss: BaseLoss = ...
    n_iter_: int = ...
    intercept_: float = ...
    coef_: ndarray = ...

    # We allow for NewtonSolver classes for the "solver" parameter but do not
    # make them public in the docstrings. This facilitates testing and
    # benchmarking.
    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        alpha: Float = 1.0,
        fit_intercept: bool = True,
        solver: Literal["lbfgs", "newton-cholesky", "lbfgs"] = "lbfgs",
        max_iter: Int = 100,
        tol: Float = 1e-4,
        warm_start: bool = False,
        verbose: Int = 0,
    ) -> None:
        ...

    def fit(
        self: _GeneralizedLinearRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> _GeneralizedLinearRegressor_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def score(
        self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...

    # TODO(1.3): remove
    @deprecated(  # type: ignore
        "Attribute `family` was deprecated in version 1.1 and will be removed in 1.3."
    )
    def family(self):
        ...


class PoissonRegressor(_GeneralizedLinearRegressor):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: float = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        alpha: Float = 1.0,
        fit_intercept: bool = True,
        solver: Literal["lbfgs", "newton-cholesky", "lbfgs"] = "lbfgs",
        max_iter: Int = 100,
        tol: Float = 1e-4,
        warm_start: bool = False,
        verbose: Int = 0,
    ) -> None:
        ...


class GammaRegressor(_GeneralizedLinearRegressor):
    feature_names_in_: ndarray = ...
    n_iter_: int = ...
    n_features_in_: int = ...
    intercept_: float = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        alpha: Float = 1.0,
        fit_intercept: bool = True,
        solver: Literal["lbfgs", "newton-cholesky", "lbfgs"] = "lbfgs",
        max_iter: Int = 100,
        tol: Float = 1e-4,
        warm_start: bool = False,
        verbose: Int = 0,
    ) -> None:
        ...


class TweedieRegressor(_GeneralizedLinearRegressor):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    intercept_: float = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        power: Float = 0.0,
        alpha: Float = 1.0,
        fit_intercept: bool = True,
        link: Literal["auto", "identity", "log", "auto"] = "auto",
        solver: Literal["lbfgs", "newton-cholesky", "lbfgs"] = "lbfgs",
        max_iter: Int = 100,
        tol: Float = 1e-4,
        warm_start: bool = False,
        verbose: Int = 0,
    ) -> None:
        ...



from typing import Literal
from numpy.random import RandomState
from ...base import BaseEstimator
from matplotlib.axes import Axes
from matplotlib.artist import Artist
from ...utils import (
    check_matplotlib_support as check_matplotlib_support,
    check_random_state as check_random_state,
)
from ..._typing import ArrayLike, MatrixLike
from matplotlib.figure import Figure
import numbers

import numpy as np


class PredictionErrorDisplay:
    figure_: Figure = ...
    ax_: Axes = ...
    scatter_: Artist = ...
    errors_lines_: Artist | None = ...
    line_: Artist = ...

    def __init__(self, *, y_true: ArrayLike, y_pred: ArrayLike) -> None:
        ...

    def plot(
        self,
        ax: None | Axes = None,
        *,
        kind: Literal[
            "actual_vs_predicted", "residual_vs_predicted", "residual_vs_predicted"
        ] = "residual_vs_predicted",
        scatter_kwargs: None | dict = None,
        line_kwargs: None | dict = None,
    ) -> PredictionErrorDisplay:
        ...

    @classmethod
    def from_estimator(
        cls,
        estimator: BaseEstimator,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        *,
        kind: Literal[
            "actual_vs_predicted", "residual_vs_predicted", "residual_vs_predicted"
        ] = "residual_vs_predicted",
        subsample: float | None | int = 1_000,
        random_state: None | RandomState | int = None,
        ax: None | Axes = None,
        scatter_kwargs: None | dict = None,
        line_kwargs: None | dict = None,
    ) -> PredictionErrorDisplay:
        ...

    @classmethod
    def from_predictions(
        cls,
        y_true: ArrayLike,
        y_pred: ArrayLike,
        *,
        kind: Literal[
            "actual_vs_predicted", "residual_vs_predicted", "residual_vs_predicted"
        ] = "residual_vs_predicted",
        subsample: float | None | int = 1_000,
        random_state: None | RandomState | int = None,
        ax: None | Axes = None,
        scatter_kwargs: None | dict = None,
        line_kwargs: None | dict = None,
    ) -> PredictionErrorDisplay:
        ...



from ._split import (
    BaseCrossValidator as BaseCrossValidator,
    BaseShuffleSplit as BaseShuffleSplit,
    KFold as KFold,
    GroupKFold as GroupKFold,
    StratifiedKFold as StratifiedKFold,
    TimeSeriesSplit as TimeSeriesSplit,
    LeaveOneGroupOut as LeaveOneGroupOut,
    LeaveOneOut as LeaveOneOut,
    LeavePGroupsOut as LeavePGroupsOut,
    LeavePOut as LeavePOut,
    RepeatedKFold as RepeatedKFold,
    RepeatedStratifiedKFold as RepeatedStratifiedKFold,
    ShuffleSplit as ShuffleSplit,
    GroupShuffleSplit as GroupShuffleSplit,
    StratifiedShuffleSplit as StratifiedShuffleSplit,
    StratifiedGroupKFold as StratifiedGroupKFold,
    PredefinedSplit as PredefinedSplit,
    train_test_split as train_test_split,
    check_cv as check_cv,
)
from ._search_successive_halving import (
    HalvingGridSearchCV as HalvingGridSearchCV,
    HalvingRandomSearchCV as HalvingRandomSearchCV,
)
from ._validation import (
    cross_val_score as cross_val_score,
    cross_val_predict as cross_val_predict,
    cross_validate as cross_validate,
    learning_curve as learning_curve,
    permutation_test_score as permutation_test_score,
    validation_curve as validation_curve,
)
from ._search import (
    GridSearchCV as GridSearchCV,
    RandomizedSearchCV as RandomizedSearchCV,
    ParameterGrid as ParameterGrid,
    ParameterSampler as ParameterSampler,
)
from ._plot import LearningCurveDisplay as LearningCurveDisplay
import typing as typing


__all__ = [
    "BaseCrossValidator",
    "BaseShuffleSplit",
    "GridSearchCV",
    "TimeSeriesSplit",
    "KFold",
    "GroupKFold",
    "GroupShuffleSplit",
    "LeaveOneGroupOut",
    "LeaveOneOut",
    "LeavePGroupsOut",
    "LeavePOut",
    "RepeatedKFold",
    "RepeatedStratifiedKFold",
    "ParameterGrid",
    "ParameterSampler",
    "PredefinedSplit",
    "RandomizedSearchCV",
    "ShuffleSplit",
    "StratifiedKFold",
    "StratifiedGroupKFold",
    "StratifiedShuffleSplit",
    "check_cv",
    "cross_val_predict",
    "cross_val_score",
    "cross_validate",
    "learning_curve",
    "LearningCurveDisplay",
    "permutation_test_score",
    "train_test_split",
    "validation_curve",
]



__all__ = ["read_gpickle", "write_gpickle"]

import pickle
import warnings

from ..classes.graph import Graph
from ..utils import open_file

def write_gpickle(G: Graph, path, protocol=...): ...
def read_gpickle(path): ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy import linalg as linalg
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral, Real as Real
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .._typing import MatrixLike, Int, Float
from ..utils import (
    check_array as check_array,
    as_float_array as as_float_array,
    check_random_state as check_random_state,
)
from ..utils.validation import check_is_fitted as check_is_fitted

FastICA_Self = TypeVar("FastICA_Self", bound="FastICA")


# Authors: Pierre Lafaye de Micheaux, Stefan van der Walt, Gael Varoquaux,
#          Bertrand Thirion, Alexandre Gramfort, Denis A. Engemann
# License: BSD 3 clause

import warnings

import numpy as np

__all__ = ["fastica", "FastICA"]


def fastica(
    X: MatrixLike,
    n_components: None | Int = None,
    *,
    algorithm: Literal["parallel", "deflation", "parallel"] = "parallel",
    whiten: str | bool = "warn",
    fun: Literal["logcosh", "exp", "cube", "logcosh"] | Callable = "logcosh",
    fun_args: None | dict = None,
    max_iter: Int = 200,
    tol: Float = 1e-04,
    w_init: None | MatrixLike = None,
    whiten_solver: Literal["eigh", "svd", "svd"] = "svd",
    random_state: RandomState | None | Int = None,
    return_X_mean: bool = False,
    compute_sources: bool = True,
    return_n_iter: bool = False,
) -> tuple[ndarray | None, ndarray, ndarray | None, ndarray, int]:
    ...


class FastICA(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    whitening_: ndarray = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    mean_: ndarray = ...
    mixing_: ndarray = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        algorithm: Literal["parallel", "deflation", "parallel"] = "parallel",
        whiten: str | bool = "warn",
        fun: Literal["logcosh", "exp", "cube", "logcosh"] | Callable = "logcosh",
        fun_args: None | dict = None,
        max_iter: Int = 200,
        tol: Float = 1e-4,
        w_init: None | MatrixLike = None,
        whiten_solver: Literal["eigh", "svd", "svd"] = "svd",
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit_transform(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...

    def fit(self: FastICA_Self, X: MatrixLike, y: Any = None) -> FastICA_Self:
        ...

    def transform(self, X: MatrixLike, copy: bool = True) -> ndarray:
        ...

    def inverse_transform(self, X: MatrixLike, copy: bool = True) -> ndarray:
        ...



"""
This type stub file was generated by pyright.
"""

from IPython.testing.skipdoctest import skip_doctest

"""Top-level display functions for displaying object in different formats."""
__all__ = ['display_pretty', 'display_html', 'display_markdown', 'display_svg', 'display_png', 'display_jpeg', 'display_latex', 'display_json', 'display_javascript', 'display_pdf', 'DisplayObject', 'TextDisplayObject', 'Pretty', 'HTML', 'Markdown', 'Math', 'Latex', 'SVG', 'ProgressBar', 'JSON', 'GeoJSON', 'Javascript', 'Image', 'set_matplotlib_formats', 'set_matplotlib_close', 'Video']
_deprecated_names = ...
__all__ = __all__ + _deprecated_names
def __getattr__(name):
    ...

def display_pretty(*objs, **kwargs) -> None:
    """Display the pretty (default) representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw text data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_html(*objs, **kwargs) -> None:
    """Display the HTML representation of an object.

    Note: If raw=False and the object does not have a HTML
    representation, no HTML will be shown.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw HTML data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_markdown(*objs, **kwargs) -> None:
    """Displays the Markdown representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw markdown data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_svg(*objs, **kwargs) -> None:
    """Display the SVG representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw svg data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_png(*objs, **kwargs) -> None:
    """Display the PNG representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw png data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_jpeg(*objs, **kwargs) -> None:
    """Display the JPEG representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw JPEG data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_latex(*objs, **kwargs) -> None:
    """Display the LaTeX representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw latex data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_json(*objs, **kwargs) -> None:
    """Display the JSON representation of an object.

    Note that not many frontends support displaying JSON.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw json data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_javascript(*objs, **kwargs) -> None:
    """Display the Javascript representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw javascript data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

def display_pdf(*objs, **kwargs) -> None:
    """Display the PDF representation of an object.

    Parameters
    ----------
    *objs : object
        The Python objects to display, or if raw=True raw javascript data to
        display.
    raw : bool
        Are the data objects raw data or Python objects that need to be
        formatted before display? [default: False]
    metadata : dict (optional)
        Metadata to be associated with the specific mimetype output.
    """
    ...

class DisplayObject:
    """An object that wraps data to be displayed."""
    _read_flags = ...
    _show_mem_addr = ...
    metadata = ...
    def __init__(self, data=..., url=..., filename=..., metadata=...) -> None:
        """Create a display object given raw data.

        When this object is returned by an expression or passed to the
        display function, it will result in the data being displayed
        in the frontend. The MIME type of the data should match the
        subclasses used, so the Png subclass should be used for 'image/png'
        data. If the data is a URL, the data will first be downloaded
        and then displayed. If

        Parameters
        ----------
        data : unicode, str or bytes
            The raw data or a URL or file to load the data from
        url : unicode
            A URL to download the data from.
        filename : unicode
            Path to a local file to load the data from.
        metadata : dict
            Dict of metadata associated to be the object when displayed
        """
        ...
    
    def __repr__(self):
        ...
    
    def reload(self): # -> None:
        """Reload the raw data from file or URL."""
        ...
    


class TextDisplayObject(DisplayObject):
    """Create a text display object given raw data.

    Parameters
    ----------
    data : str or unicode
        The raw data or a URL or file to load the data from.
    url : unicode
        A URL to download the data from.
    filename : unicode
        Path to a local file to load the data from.
    metadata : dict
        Dict of metadata associated to be the object when displayed
    """
    ...


class Pretty(TextDisplayObject):
    ...


class HTML(TextDisplayObject):
    def __init__(self, data=..., url=..., filename=..., metadata=...) -> None:
        ...
    
    def __html__(self): # -> None:
        """
        This method exists to inform other HTML-using modules (e.g. Markupsafe,
        htmltag, etc) that this object is HTML and does not need things like
        special characters (<>&) escaped.
        """
        ...
    


class Markdown(TextDisplayObject):
    ...


class Math(TextDisplayObject):
    ...


class Latex(TextDisplayObject):
    ...


class SVG(DisplayObject):
    """Embed an SVG into the display.

    Note if you just want to view a svg image via a URL use `:class:Image` with
    a url=URL keyword argument.
    """
    _read_flags = ...
    _data = ...
    @property
    def data(self): # -> None:
        ...
    
    @data.setter
    def data(self, svg): # -> None:
        ...
    


class ProgressBar(DisplayObject):
    """Progressbar supports displaying a progressbar like element
    """
    def __init__(self, total) -> None:
        """Creates a new progressbar

        Parameters
        ----------
        total : int
            maximum size of the progressbar
        """
        ...
    
    def __repr__(self):
        ...
    
    def display(self): # -> None:
        ...
    
    def update(self): # -> None:
        ...
    
    @property
    def progress(self):
        ...
    
    @progress.setter
    def progress(self, value): # -> None:
        ...
    
    def __iter__(self): # -> Self@ProgressBar:
        ...
    
    def __next__(self):
        """Returns current value and increments display by one."""
        ...
    


class JSON(DisplayObject):
    """JSON expects a JSON-able dict or list

    not an already-serialized JSON string.

    Scalar types (None, number, string) are not allowed, only dict or list containers.
    """
    _data = ...
    def __init__(self, data=..., url=..., filename=..., expanded=..., metadata=..., root=..., **kwargs) -> None:
        """Create a JSON display object given raw data.

        Parameters
        ----------
        data : dict or list
            JSON data to display. Not an already-serialized JSON string.
            Scalar types (None, number, string) are not allowed, only dict
            or list containers.
        url : unicode
            A URL to download the data from.
        filename : unicode
            Path to a local file to load the data from.
        expanded : boolean
            Metadata to control whether a JSON display component is expanded.
        metadata : dict
            Specify extra metadata to attach to the json display object.
        root : str
            The name of the root element of the JSON tree
        """
        ...
    
    @property
    def data(self): # -> None:
        ...
    
    @data.setter
    def data(self, data): # -> None:
        ...
    


_css_t = ...
_lib_t1 = ...
_lib_t2 = ...
class GeoJSON(JSON):
    """GeoJSON expects JSON-able dict

    not an already-serialized JSON string.

    Scalar types (None, number, string) are not allowed, only dict containers.
    """
    def __init__(self, *args, **kwargs) -> None:
        """Create a GeoJSON display object given raw data.

        Parameters
        ----------
        data : dict or list
            VegaLite data. Not an already-serialized JSON string.
            Scalar types (None, number, string) are not allowed, only dict
            or list containers.
        url_template : string
            Leaflet TileLayer URL template: http://leafletjs.com/reference.html#url-template
        layer_options : dict
            Leaflet TileLayer options: http://leafletjs.com/reference.html#tilelayer-options
        url : unicode
            A URL to download the data from.
        filename : unicode
            Path to a local file to load the data from.
        metadata : dict
            Specify extra metadata to attach to the json display object.

        Examples
        --------
        The following will display an interactive map of Mars with a point of
        interest on frontend that do support GeoJSON display.

            >>> from IPython.display import GeoJSON

            >>> GeoJSON(data={
            ...     "type": "Feature",
            ...     "geometry": {
            ...         "type": "Point",
            ...         "coordinates": [-81.327, 296.038]
            ...     }
            ... },
            ... url_template="http://s3-eu-west-1.amazonaws.com/whereonmars.cartodb.net/{basemap_id}/{z}/{x}/{y}.png",
            ... layer_options={
            ...     "basemap_id": "celestia_mars-shaded-16k_global",
            ...     "attribution" : "Celestia/praesepe",
            ...     "minZoom" : 0,
            ...     "maxZoom" : 18,
            ... })
            <IPython.core.display.GeoJSON object>

        In the terminal IPython, you will only see the text representation of
        the GeoJSON object.

        """
        ...
    


class Javascript(TextDisplayObject):
    def __init__(self, data=..., url=..., filename=..., lib=..., css=...) -> None:
        """Create a Javascript display object given raw data.

        When this object is returned by an expression or passed to the
        display function, it will result in the data being displayed
        in the frontend. If the data is a URL, the data will first be
        downloaded and then displayed.

        In the Notebook, the containing element will be available as `element`,
        and jQuery will be available.  Content appended to `element` will be
        visible in the output area.

        Parameters
        ----------
        data : unicode, str or bytes
            The Javascript source code or a URL to download it from.
        url : unicode
            A URL to download the data from.
        filename : unicode
            Path to a local file to load the data from.
        lib : list or str
            A sequence of Javascript library URLs to load asynchronously before
            running the source code. The full URLs of the libraries should
            be given. A single Javascript library URL can also be given as a
            string.
        css : list or str
            A sequence of css files to load before running the source code.
            The full URLs of the css files should be given. A single css URL
            can also be given as a string.
        """
        ...
    


_PNG = ...
_JPEG = ...
class Image(DisplayObject):
    _read_flags = ...
    _FMT_JPEG = ...
    _FMT_PNG = ...
    _FMT_GIF = ...
    _ACCEPTABLE_EMBEDDINGS = ...
    _MIMETYPES = ...
    def __init__(self, data=..., url=..., filename=..., format=..., embed=..., width=..., height=..., retina=..., unconfined=..., metadata=..., alt=...) -> None:
        """Create a PNG/JPEG/GIF image object given raw data.

        When this object is returned by an input cell or passed to the
        display function, it will result in the image being displayed
        in the frontend.

        Parameters
        ----------
        data : unicode, str or bytes
            The raw image data or a URL or filename to load the data from.
            This always results in embedded image data.

        url : unicode
            A URL to download the data from. If you specify `url=`,
            the image data will not be embedded unless you also specify `embed=True`.

        filename : unicode
            Path to a local file to load the data from.
            Images from a file are always embedded.

        format : unicode
            The format of the image data (png/jpeg/jpg/gif). If a filename or URL is given
            for format will be inferred from the filename extension.

        embed : bool
            Should the image data be embedded using a data URI (True) or be
            loaded using an <img> tag. Set this to True if you want the image
            to be viewable later with no internet connection in the notebook.

            Default is `True`, unless the keyword argument `url` is set, then
            default value is `False`.

            Note that QtConsole is not able to display images if `embed` is set to `False`

        width : int
            Width in pixels to which to constrain the image in html

        height : int
            Height in pixels to which to constrain the image in html

        retina : bool
            Automatically set the width and height to half of the measured
            width and height.
            This only works for embedded images because it reads the width/height
            from image data.
            For non-embedded images, you can just set the desired display width
            and height directly.

        unconfined : bool
            Set unconfined=True to disable max-width confinement of the image.

        metadata : dict
            Specify extra metadata to attach to the image.

        alt : unicode
            Alternative text for the image, for use by screen readers.

        Examples
        --------
        embedded image data, works in qtconsole and notebook
        when passed positionally, the first arg can be any of raw image data,
        a URL, or a filename from which to load image data.
        The result is always embedding image data for inline images.

        >>> Image('https://www.google.fr/images/srpr/logo3w.png') # doctest: +SKIP
        <IPython.core.display.Image object>

        >>> Image('/path/to/image.jpg')
        <IPython.core.display.Image object>

        >>> Image(b'RAW_PNG_DATA...')
        <IPython.core.display.Image object>

        Specifying Image(url=...) does not embed the image data,
        it only generates ``<img>`` tag with a link to the source.
        This will not work in the qtconsole or offline.

        >>> Image(url='https://www.google.fr/images/srpr/logo3w.png')
        <IPython.core.display.Image object>

        """
        ...
    
    def reload(self): # -> None:
        """Reload the raw data from file or URL."""
        ...
    


class Video(DisplayObject):
    def __init__(self, data=..., url=..., filename=..., embed=..., mimetype=..., width=..., height=..., html_attributes=...) -> None:
        """Create a video object given raw data or an URL.

        When this object is returned by an input cell or passed to the
        display function, it will result in the video being displayed
        in the frontend.

        Parameters
        ----------
        data : unicode, str or bytes
            The raw video data or a URL or filename to load the data from.
            Raw data will require passing ``embed=True``.

        url : unicode
            A URL for the video. If you specify ``url=``,
            the image data will not be embedded.

        filename : unicode
            Path to a local file containing the video.
            Will be interpreted as a local URL unless ``embed=True``.

        embed : bool
            Should the video be embedded using a data URI (True) or be
            loaded using a <video> tag (False).

            Since videos are large, embedding them should be avoided, if possible.
            You must confirm embedding as your intention by passing ``embed=True``.

            Local files can be displayed with URLs without embedding the content, via::

                Video('./video.mp4')

        mimetype : unicode
            Specify the mimetype for embedded videos.
            Default will be guessed from file extension, if available.

        width : int
            Width in pixels to which to constrain the video in HTML.
            If not supplied, defaults to the width of the video.

        height : int
            Height in pixels to which to constrain the video in html.
            If not supplied, defaults to the height of the video.

        html_attributes : str
            Attributes for the HTML ``<video>`` block.
            Default: ``"controls"`` to get video controls.
            Other examples: ``"controls muted"`` for muted video with controls,
            ``"loop autoplay"`` for looping autoplaying video without controls.

        Examples
        --------
        ::

            Video('https://archive.org/download/Sita_Sings_the_Blues/Sita_Sings_the_Blues_small.mp4')
            Video('path/to/video.mp4')
            Video('path/to/video.mp4', embed=True)
            Video('path/to/video.mp4', embed=True, html_attributes="controls muted autoplay")
            Video(b'raw-videodata', embed=True)
        """
        ...
    
    def reload(self): # -> None:
        ...
    


@skip_doctest
def set_matplotlib_formats(*formats, **kwargs): # -> None:
    """
    .. deprecated:: 7.23

       use `matplotlib_inline.backend_inline.set_matplotlib_formats()`

    Select figure formats for the inline backend. Optionally pass quality for JPEG.

    For example, this enables PNG and JPEG output with a JPEG quality of 90%::

        In [1]: set_matplotlib_formats('png', 'jpeg', quality=90)

    To set this in your config files use the following::

        c.InlineBackend.figure_formats = {'png', 'jpeg'}
        c.InlineBackend.print_figure_kwargs.update({'quality' : 90})

    Parameters
    ----------
    *formats : strs
        One or more figure formats to enable: 'png', 'retina', 'jpeg', 'svg', 'pdf'.
    **kwargs
        Keyword args will be relayed to ``figure.canvas.print_figure``.
    """
    ...

@skip_doctest
def set_matplotlib_close(close=...): # -> None:
    """
    .. deprecated:: 7.23

        use `matplotlib_inline.backend_inline.set_matplotlib_close()`

    Set whether the inline backend closes all figures automatically or not.

    By default, the inline backend used in the IPython Notebook will close all
    matplotlib figures automatically after each cell is run. This means that
    plots in different cells won't interfere. Sometimes, you may want to make
    a plot in one cell and then refine it in later cells. This can be accomplished
    by::

        In [1]: set_matplotlib_close(False)

    To set this in your config files use the following::

        c.InlineBackend.close_figures = False

    Parameters
    ----------
    close : bool
        Should all matplotlib figures be automatically closed after each cell is
        run?
    """
    ...




from typing import Sequence
from .axes import Axes
from types import SimpleNamespace

RIGHT = ...
UP = ...
DOWN = ...

class Sankey:
    def __init__(
        self,
        ax: Axes | None = ...,
        scale: float = ...,
        unit: str = ...,
        format: str = ...,
        gap: float = ...,
        radius: float = ...,
        shoulder: float = ...,
        offset: float = ...,
        head_angle: int = ...,
        margin: float = ...,
        tolerance: float = ...,
        **kwargs
    ) -> None: ...
    def add(
        self,
        patchlabel: str = ...,
        flows: Sequence[float] = ...,
        orientations: Sequence[int] = ...,
        labels: Sequence[str | None] = ...,
        trunklength: float = ...,
        pathlengths: Sequence[float] = ...,
        prior: int = ...,
        connect: Sequence[int] = ...,
        rotation: float = ...,
        **kwargs
    ) -> Sankey: ...
    def finish(self) -> list[SimpleNamespace]: ...



from typing import Callable, ClassVar, Iterable, Literal, TypeVar
from ..model_selection import BaseCrossValidator
from ..base import BaseEstimator
from ._base import SelectorMixin
from ..model_selection._split import BaseShuffleSplit
from ..utils._param_validation import (
    HasMethods as HasMethods,
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from ..base import MetaEstimatorMixin, clone as clone
from ..model_selection import cross_val_score as cross_val_score
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..metrics import get_scorer_names as get_scorer_names
from ..utils.validation import check_is_fitted as check_is_fitted

SequentialFeatureSelector_Self = TypeVar(
    "SequentialFeatureSelector_Self", bound="SequentialFeatureSelector"
)


import numpy as np

import warnings


class SequentialFeatureSelector(SelectorMixin, MetaEstimatorMixin, BaseEstimator):
    support_: ndarray = ...
    n_features_to_select_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: BaseEstimator,
        *,
        n_features_to_select: float | int | Literal["auto", "warn"] = "warn",
        tol: None | Float = None,
        direction: Literal["forward", "backward", "forward"] = "forward",
        scoring: None | str | Callable = None,
        cv: Iterable | int | BaseShuffleSplit | BaseCrossValidator = 5,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: SequentialFeatureSelector_Self, X: MatrixLike, y: None | ArrayLike = None
    ) -> SequentialFeatureSelector_Self:
        ...



from typing import Any, Callable, List, Optional, Tuple, Union

NONE_ID: Any
NO_RECEIVERS: Any

class Signal:
    receivers: Any = ...
    providing_args: Any = ...
    lock: Any = ...
    use_caching: Any = ...
    sender_receivers_cache: Any = ...
    def __init__(
        self, providing_args: List[str] = ..., use_caching: bool = ...
    ) -> None: ...
    def connect(
        self,
        receiver: Callable[..., Any],
        sender: Optional[object] = ...,
        weak: bool = ...,
        dispatch_uid: Optional[str] = ...,
    ) -> None: ...
    def disconnect(
        self,
        receiver: Optional[Callable[..., Any]] = ...,
        sender: Optional[object] = ...,
        dispatch_uid: Optional[str] = ...,
    ) -> bool: ...
    def has_listeners(self, sender: Any = ...) -> bool: ...
    def send(
        self, sender: Any, **named: Any
    ) -> List[Tuple[Callable[..., Any], Optional[str]]]: ...
    def send_robust(
        self, sender: Any, **named: Any
    ) -> List[Tuple[Callable[..., Any], Union[ValueError, str]]]: ...

def receiver(
    signal: Union[List[Signal], Signal], **kwargs: Any
) -> Callable[..., Any]: ...



from typing import Mapping, Any

from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = ["random_internet_as_graph"]

def uniform_int_from_avg(a, m, seed): ...
def choose_pref_attach(degs: Mapping, seed) -> Any: ...

class AS_graph_generator:
    def __init__(self, n, seed): ...
    def t_graph(self): ...
    def add_edge(self, i, j, kind): ...
    def choose_peer_pref_attach(self, node_list): ...
    def choose_node_pref_attach(self, node_list): ...
    def add_customer(self, i, j): ...
    def add_node(
        self, i: Any, kind: str, reg2prob: float, avg_deg: float, t_edge_prob: float
    ) -> Any: ...
    def add_m_peering_link(self, m: Any, to_kind: str) -> bool: ...
    def add_cp_peering_link(self, cp: Any, to_kind: str) -> bool: ...
    def graph_regions(self, rn): ...
    def add_peering_links(self, from_kind, to_kind): ...
    def generate(self): ...

@py_random_state(1)
def random_internet_as_graph(n, seed=None): ...



from typing import Any, Callable, ClassVar, Literal, Mapping, TypeVar
from ._kmeans import k_means as k_means
from numpy.random import RandomState
from ..manifold import spectral_embedding as spectral_embedding
from scipy.sparse._coo import coo_matrix
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.linalg import LinAlgError as LinAlgError, qr as qr, svd as svd
from numbers import Integral as Integral, Real as Real
from ..neighbors import (
    kneighbors_graph as kneighbors_graph,
    NearestNeighbors as NearestNeighbors,
)
from scipy.sparse import csc_matrix as csc_matrix
from ..base import BaseEstimator, ClusterMixin
from ..metrics.pairwise import (
    pairwise_kernels as pairwise_kernels,
    KERNEL_PARAMS as KERNEL_PARAMS,
)
from .._typing import MatrixLike, Int, Float, ArrayLike
from ..utils import (
    check_random_state as check_random_state,
    as_float_array as as_float_array,
)

SpectralClustering_Self = TypeVar("SpectralClustering_Self", bound="SpectralClustering")

import warnings

import numpy as np


def cluster_qr(vectors: MatrixLike) -> ndarray:
    ...


def discretize(
    vectors: MatrixLike,
    *,
    copy: bool = True,
    max_svd_restarts: Int = 30,
    n_iter_max: Int = 20,
    random_state: RandomState | None | Int = None,
) -> ndarray:
    ...


def spectral_clustering(
    affinity: coo_matrix | MatrixLike,
    *,
    n_clusters: Int = 8,
    n_components: None | Int = None,
    eigen_solver: Literal["arpack", "lobpcg", "amg"] | None = None,
    random_state: RandomState | None | Int = None,
    n_init: Int = 10,
    eigen_tol: str | Float = "auto",
    assign_labels: Literal["kmeans", "discretize", "cluster_qr", "kmeans"] = "kmeans",
    verbose: bool = False,
) -> ndarray:
    ...


class SpectralClustering(ClusterMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    labels_: ndarray = ...
    affinity_matrix_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_clusters: Int = 8,
        *,
        eigen_solver: None | Literal["arpack", "lobpcg", "amg"] = None,
        n_components: None | Int = None,
        random_state: RandomState | None | Int = None,
        n_init: Int = 10,
        gamma: Float = 1.0,
        affinity: str | Callable = "rbf",
        n_neighbors: Int = 10,
        eigen_tol: str | Float = "auto",
        assign_labels: Literal[
            "kmeans", "discretize", "cluster_qr", "kmeans"
        ] = "kmeans",
        degree: Float = 3,
        coef0: Float = 1,
        kernel_params: None | Mapping[str, Any] = None,
        n_jobs: None | Int = None,
        verbose: bool = False,
    ) -> None:
        ...

    def fit(
        self: SpectralClustering_Self, X: MatrixLike, y: Any = None
    ) -> SpectralClustering_Self:
        ...

    def fit_predict(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...
